/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/actions', [
    'io.ox/core/extPatterns/links',
    'io.ox/core/http',
    'io.ox/core/notifications',
    'io.ox/contacts/util',
    'io.ox/contacts/api',
    'settings!io.ox/mail',
    'settings!io.ox/core',
    'com.voiceworks/ox-messenger/core/views/ext/add-participant',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/mediator',
    'com.voiceworks/ox-messenger/conversation/models/groupconversation',
    'com.voiceworks/ox-messenger/util',
    'com.voiceworks/ox-messenger/roster/main',
    'com.voiceworks/ox-messenger/history/main',
    'gettext!com.voiceworks/ox-messenger',
    'settings!com.voiceworks/ox-messenger'
], function (links, http, notifications, ContactsUtil, ContactsApi, MailSettings, CoreSettings, AddParticipant,
             Aggregator, Mediator, GroupConversationModel, Util, Roster, History, gt, settings) {
    'use strict';

    var Action = links.Action;

    new Action('com.voiceworks/ox-messenger/actions/start-group-conversation', {
        capabilities: 'messenger-group',
        action: function (baton) {
            require(['com.voiceworks/ox-messenger/conversation/group/main'], function (m) {
                m.getApp().launch();
            });
        }
    });

    new Action('com.voiceworks/ox-messenger/actions/addbuddy', {
        action: function (baton) {
            var pNode = $('<div class="input add-buddy-field">').append(
                    $('<input tabindex="1" type="text" class="form-control add-buddy">')
                );

            // late require because actions had to be loaded first
            require([
                'io.ox/core/tk/dialogs',
                'com.voiceworks/ox-messenger/roster/main',
                'com.voiceworks/ox-messenger/mediator'
            ], function (dialogs, Roster, Mediator) {
                var dialog = new dialogs.ModalDialog({ width: 350, height: 100, async: true, 'tabTrap': true });
                dialog.header($('<h4>').text(gt('Add to buddy list')))
                    //#. Button: invite a new buddy
                    .addPrimaryButton('invite', gt('Invite'), 'invite', { tabIndex: '1' })
                    .addButton('cancel', gt('Cancel'), 'cancel', { tabIndex: '1' })
                    .on('invite', function () {
                        inviteUser();
                    })
                    .append(
                    $('<div>').append(
                        $('<div class="item-list row-fluid">'),
                        gt('Email address'),
                        pNode
                    ).addClass('control-group').css({ width: '320px', height: '75px' })
                )
                .show(function () {
                    this.find('.add-buddy').focus();
                });

                function inviteUser() {
                    var email = $('input.add-buddy').val();
                    $('.control-group').removeClass('error');
                    $('.help-inline').each(function () {
                        $(this).remove();
                    });

                    if ($.trim(email) === '') {
                        $('.add-buddy-field').append(
                            $('<span>').text(gt('Field must be filled')).addClass('help-inline errortext')
                        ).parent().addClass('error');
                        dialog.idle();
                        return false;
                    }

                    var regEmail = /\@/.test(email);
                    if (!regEmail) {
                        $('.add-buddy-field').append(
                            $('<span>').text(gt('This is not a valid email address')).addClass('help-inline errortext')
                        ).parent().addClass('error');
                        dialog.idle();
                        return false;
                    }

                    if (Roster.getUserById(email).get('inRoster')) {
                        $('.add-buddy-field').append(
                            $('<span>').text(gt('%s is already a buddy', email)).addClass('help-inline errortext')
                        ).parent().addClass('error');
                        dialog.idle();
                        return false;
                    }

                    if (Mediator.inviteUser(email)) {
                        //#. Invitation to add someone as a buddy
                        notifications.yell('success', gt('Invitation to user %s has been sent', email));

                        Mediator.addContact(email)
                            .fail(function() {
                                notifications.yell('error', gt('Error creating contact for %s', email));
                            });

                        dialog.close();
                        return true;
                    } else {
                        //#. Buddy invitation could not be sent / buddy could not be invited
                        notifications.yell('error', gt('Error adding %s', email));
                        dialog.idle();
                        return false;
                    }
                }
            });
        }
    });

    new Action('com.voiceworks/ox-messenger/actions/conversation/delete', {
        id: 'delete',
        requires: function (e) {
            var selectedConversations = e.baton.app.listControl.listView.selection.get();

            return selectedConversations.length > 0;
        },
        multiple: function (list, e) {
            var selectedConversations = e.app.listControl.listView.selection.get();

            if (selectedConversations.length === 0){
                return;
            }

            require(['io.ox/core/tk/dialogs'], function (dialogs) {
                var question = gt.ngettext(
                    'Do you want to permanently delete this conversation?',
                    'Do you want to permanently delete these conversations?',
                    selectedConversations.length
                );

                new dialogs.ModalDialog()
                    .text(question)
                    .addPrimaryButton('delete', gt('Delete'), 'delete', { tabIndex: '1' })
                    .addButton('cancel', gt('Cancel'), 'cancel', { tabIndex: '1' })
                    .show()
                    .done(function (action) {
                        if (action === 'delete') {
                            Aggregator.trigger('conversation:delete', selectedConversations);
                        }
                    });
            });
        }
    });

    new Action('com.voiceworks/ox-messenger/actions/addparticipant', {
        capabilities: 'messenger-group',
        action: function (baton) {
            var self = this;

            require([
                'io.ox/core/tk/dialogs',
                'com.voiceworks/ox-messenger/roster/main',
                'com.voiceworks/ox-messenger/core/session'
            ], function (dialogs, Roster, Session) {

                var selectedUser,
                    excludedUsers = {},
                    allBuddys = Roster.getAllUsers(),
                    dialog = new dialogs.ModalDialog({
                        width: 450,
                        async: true,
                        'tabTrap': true,
                        addclass: 'ox-messenger'
                    });

                dialog.getBody().append(
                    $('<div class="dndinfo alert alert-info">').text(gt('Please select the contact you want to add to the conversation.'))
                );

                dialog.getBody().addClass('messenger--dialog__participant-wrapper');

                // Calculate the users to be excluded
                if (!baton.isGroupCallInvite) {
                    // Do not show users already in the conversation
                    baton.data.participants.each(function (participant) {
                        excludedUsers[participant.id] = true;
                    });
                }

                var autocomplete = new AddParticipant({
                    collection: baton.data.participants,
                    exclude: excludedUsers,
                    modelParser: function(oxUser){
                        return Roster.getUserById(oxUser.get('email1'));
                    },
                    inviteBuddyInExistingConversation: function(e, model, value, view){
                        var email = model.id,
                            user = model;

                        // Existing group conversation
                        if (baton.data.isMuc()) {

                            var roomJid = baton.data.get('roomId') + '@' + baton.data.get('roomHost'),
                            conversation = Session.user.conversations.get(roomJid),
                            alreadyMember = conversation.participants.get(email);

                            if (!alreadyMember) {
                                Aggregator.trigger('group:invite', {
                                    conversation: baton.data,
                                    user: user,
                                    notify: true
                                });
                            } else if (!baton.isGroupCallInvite) {
                                var $wrapper = view.$el.parents('.messenger--dialog__participant-wrapper'),
                                    $errorSpan = $('<span class="help-block messenger--help-block has-error">').text(gt('This buddy is already participating'));

                                $wrapper.find('span.has-error').remove();
                                $wrapper.append($errorSpan);

                                dialog.idle();
                                return;
                            }

                            if (baton.isGroupCallInvite) {
                                conversation.groupCallInvitees.add(user);
                                conversation.sendGroupCallInvite(user);
                            }

                            dialog.close();

                            return;
                        }

                        // Start new group conversation
                        var convModel = new GroupConversationModel({
                            user: Session.user,
                            roomHost: settings.get('groupConversationRoomHost'),
                            roomId: Util.generateUUID()
                        });

                        convModel.set('id', convModel.get('roomId') + '@' + convModel.get('roomHost').toLowerCase());
                        convModel.set('timestamp', new Date().getTime());
                        convModel.set('lastmsg', gt('Group started'));
                        convModel.set('myRoom', true);

                        //#. Generated group conversation name, example: John Smith's chat
                        var roomName = gt('%s\'s chat',
                            Session.user.getDisplayName()
                        );

                        // Shorten te room name if it is too long
                        if (roomName.length > convModel.maxRoomNameLength) {
                            var roomNameSuffixLength = gt('%s\'s chat', '').length + 3;

                            roomName = gt('%s\'s chat',
                                Util.stringTruncate(Session.user.getDisplayName(), convModel.maxRoomNameLength - roomNameSuffixLength, '...')
                            );
                        }

                        convModel.set('roomName', roomName);
                        convModel.set('title', roomName);

                        var participants = [];

                        participants.push(user);
                        baton.data.getOtherParticipants().each(function (participant) {
                            participants.push(participant);
                        });

                        // Check for errors
                        var convError = convModel.validate();
                        if (convError) {
                            notifications.yell('error', gt('Could not create new group conversation: ') + convError);
                            dialog.idle();
                            convModel.destroy();
                            return false;
                        }

                        // Create group and invite participants
                        Aggregator.trigger('group:create', { conversation: convModel, participants: participants });

                        // Add to conversations
                        Session.user.conversations.add(convModel);

                        // Show new conversation
                        Aggregator.trigger('conversation:show', {id : convModel.get('id'), type: 'chat' });

                        Aggregator.trigger('groupconversation:changed', {});

                        notifications.yell('info', gt('A new group conversation has been started')); // removed listing of participants since can be more than 2
                        dialog.close();
                        return true;
                    }
                });


                autocomplete.on('select', function (data) {
                    require([
                        'com.voiceworks/ox-messenger/roster/main',
                    ], function (Roster) {
                        var user = allBuddys.get(data.email1);

                        if (!user) {
                            notifications.yell('error', gt('You can not add this user to the conversation since the user does not appear to be in your buddylist'));
                            return false;
                        }

                        if (baton.isGroupCallInvite) {
                            var canCallWithErrors = Session.user.canCallWith(Session.user.call.get('type'), user, true);

                            if (canCallWithErrors !== true) {
                                notifications.yell('error', canCallWithErrors.join(' '));
                                return false;
                            }
                        }

                        // overwrite display_name
                        data.display_name = ContactsUtil.getMailFullName(data);
                        $('.add-participant').val(data.display_name);

                        selectedUser = data;
                    });
                });

                dialog.header($('<h4>').text(gt('' +
                    'Add people to this conversation')))
                    .addButton('cancel', gt('Cancel'), 'cancel', {tabIndex: '1'})
                    .append(
                        $('<div>').append(autocomplete.$el)
                    )
                .show();
                autocomplete.render();

                $('.add-participant').focus();
            });
        },
    });

    new Action('com.voiceworks/ox-messenger/actions/inviteguest', {
        capabilities: 'messenger-guestmode',
        action: function (baton) {
            var pNodeFirstName = $('<div class="input invite-guest-field-firstname">').append(
                    $('<input tabindex="1" type="text" class="form-control invite-guest-firstname">')
                ),
                pNodeLastName = $('<div class="input invite-guest-field-lastname">').append(
                    $('<input tabindex="1" type="text" class="form-control invite-guest-lastname">')
                ),
                pNodeEmail = $('<div class="input invite-guest-field-email">').append(
                    $('<input tabindex="1" type="text" class="form-control invite-guest-email">')
                );

            require(['io.ox/core/tk/dialogs'], function (dialogs) {
                var dialog = new dialogs.ModalDialog({ width: 350, async: true, 'tabTrap': true, enter: 'invite' });

                dialog.getBody().append(
                    $('<div class="dndinfo alert alert-info">').text(gt('Enter the name and e-mail address of an external user you want to start a conversation with.'))
                );
                dialog.header($('<h4>').text(gt('Invite guest user')))
                    .addPrimaryButton('invite', gt('Invite'), 'invite', {tabIndex: '1'})
                    .addButton('cancel', gt('Cancel'), 'cancel', {tabIndex: '1'})
                    .on('invite', function () {
                        inviteGuestUser(dialog);
                    })
                    .append(
                        $('<div>').append(
                            $('<div>').addClass('item-list row-fluid'),
                            gt('First name'),
                            pNodeFirstName
                        ),
                        $('<div>').append(
                            $('<div>').addClass('item-list row-fluid'),
                            gt('Last name'),
                            pNodeLastName
                        ),
                        $('<div>').append(
                            $('<div>').addClass('item-list row-fluid'),
                            gt('E-mail address'),
                            pNodeEmail
                        )
                    )
                .show(function () {
                    this.find('.invite-guest-firstname').focus();
                });
            });
        }
    });

    function inviteGuestUser(dialog) {
        var firstName = $('input.invite-guest-firstname').val(),
            lastName = $('input.invite-guest-lastname').val(),
            email = $('input.invite-guest-email').val();

        $('.control-group').removeClass('error');
        $('.help-inline').each(function () {
            $(this).remove();
        });

        if ($.trim(firstName) === '') {
            $('.invite-guest-field-firstname').append(
                $('<span>').text(gt('Field must be filled')).addClass('help-inline errortext')
            ).parent().addClass('error');
            dialog.idle();
            return false;
        }

        if ($.trim(lastName) === '') {
            $('.invite-guest-field-lastname').append(
                $('<span>').text(gt('Field must be filled')).addClass('help-inline errortext')
            ).parent().addClass('error');
            dialog.idle();
            return false;
        }

        if ($.trim(email) === '') {
            $('.invite-guest-field-email').append(
                $('<span>').text(gt('Field must be filled')).addClass('help-inline errortext')
            ).parent().addClass('error');
            dialog.idle();
            return false;
        }

        if (Roster.hasUser(email)) {
            $('.invite-guest-field-email').append(
                $('<span>').text(gt('%s is already a buddy', email)).addClass('help-inline errortext')
            ).parent().addClass('error');
            dialog.idle();
            return false;
        }

        var regEmail = /\@/.test(email);
        if (!regEmail) {
            $('.invite-guest-field-email').append(
                $('<span>').text(gt('This is not a valid email address')).addClass('help-inline errortext')
            ).parent().addClass('error');
            dialog.idle();
            return false;
        }

        http.GET({
            module: 'messenger',
            params: {
                action: 'create_guest_user',
                session: ox.session,
                firstName: firstName,
                lastName: lastName,
                mail: email
            }
        }).done(function (result) {
            ContactsApi.create({
                folder_id: CoreSettings.get('folder/contacts'),
                email1: result.userName,
                email2: email,
                first_name: result.firstName,
                last_name: result.lastName,
                display_name: result.firstName + ' ' + result.lastName,
                position: gt('Guest User')
            }).done(function (contact) {
                guestCreatedDialog(result, firstName, lastName, email);
            }).fail(function (e) {
                notifications.yell('error', gt('Error creating guest account contact for %s', email));
                console.error(e);
            });
        })
        .fail(function (e) {
            notifications.yell('error', gt('Error creating guest account for %s', email));
        });

        dialog.close();
    }

    function guestCreatedDialog(result, firstName, lastName, email) {
        require([
            'io.ox/core/tk/dialogs',
            'com.voiceworks/ox-messenger/core/session'
        ], function (dialogs, Session) {
            var dialogSuccess = new dialogs.ModalDialog({ width: 450, async: true, 'tabTrap': true }),
                url = result.url + '?token=' + result.token,
                correctDateTimeString = result.expiresAt.replace(/ /, 'T') + 'Z',
                expirationDate = Util.formatDateTime(new Date(correctDateTimeString).getTime());

            dialogSuccess.getBody().append(
                $('<div>').addClass('dndinfo alert').text(gt('The guest user has been created. Please send the URL below to the user you invited or create an invitation e-mail.')),
                $('<div>').addClass('alert alert-info user-select-text messenger--word-wrap').text(url),
                $('<div>').append(
                    //#. For example: This link will expire on 01-01-2020 12:13:14
                    $('<strong>').addClass('dndinfo alert').text(gt('This link will expire on %s', expirationDate))
                )
            );

            dialogSuccess.header($('<h4>').text(gt('Guest user created')))
                .addButton('cancel', gt('Close'), 'cancel', { tabIndex: '1' })
                .addPrimaryButton('email', gt('Compose invitation e-mail'), 'email', {tabIndex: '1'})
                .on('email', function () {
                    dialogSuccess.close();

                    sendGuestInviteMail(firstName, lastName, email, Session.user.getDisplayName(), url, expirationDate);
                })
            .show();

            // Fetch new roster with provisioned guest
            Aggregator.trigger('roster:add-guest', {
                username: result.userName,
                name: ContactsUtil.getMailFullName({
                    first_name: firstName,
                    last_name: lastName
                }),
                email1: result.userName
            });
        });
    }

    function sendGuestInviteMail(guestFirstName, guestLastName, guestEmail, invitorName, guestTokenUrl, tokenExirationDate) {
        //#. Guest mode invitation mail body
        var text = gt('Dear %1$s %2$s,<br /><br />You are invited for a conversation by %3$s.<br /><br />Please click the link below to start the conversation:<br /><br />%4$s<br /><br />The link will expire at %5$s<br /><br /><br />Regards,<br /><br />%3$s', guestFirstName, guestLastName, invitorName, '<a href="' + guestTokenUrl + '">' + guestTokenUrl + '</a>', tokenExirationDate),
            textplain = gt('Dear %1$s %2$s,<br /><br />You are invited for a conversation by %3$s.<br /><br />Please click the link below to start the conversation:<br /><br />%4$s<br /><br />The link will expire at %5$s<br /><br /><br />Regards,<br /><br />%3$s', guestFirstName, guestLastName, invitorName, guestTokenUrl, tokenExirationDate),
            format = MailSettings.get('messageFormat'),
            attachments = [{
                content: text,
                content_type: 'text/html',
                disp: 'inline'
            }, {
                content: textplain,
                content_type: 'text/plain',
                disp: 'inline',
                raw: true
            }];

        switch (format) {
        case 'alternative':
        case 'html':
            // Remove plaintext attachment
            attachments.pop();
            break;
        case 'text':
            // Remove html attachment
            attachments.shift();
            break;
        }

        debugger;

        var data = {
            //#. Guest mode invitation mail subject
            subject: gt('You are invited for a conversation by %s', invitorName),
            attachments: attachments,
            to: [[
                ContactsUtil.getMailFullName({
                    first_name: guestFirstName,
                    last_name: guestLastName
                }),
                guestEmail
            ]]
        };

        ox.registry.call('mail-compose', 'compose', data);
    }

    new Action('com.voiceworks/ox-messenger/actions/call-history', {
        capabilities: 'messenger-call',
        action: function (baton) {
            History.show();
        }
    });

});
