/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/group/main', [
    'io.ox/core/notifications',
    'com.voiceworks/ox-messenger/util',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/conversation/views/groupconversationform',
    'com.voiceworks/ox-messenger/conversation/models/message',
    'com.voiceworks/ox-messenger/conversation/models/groupconversation',
    'com.voiceworks/ox-messenger/roster/main',
    'com.voiceworks/ox-messenger/core/session',
    'gettext!com.voiceworks/ox-messenger',
    'settings!com.voiceworks/ox-messenger',
    'less!com.voiceworks/ox-messenger/conversation/group/style'
], function (notifications, Util, Aggregator, CreateGroupConversationView, MessageModel, GroupConversationModel, Roster, Session, gt, settings) {
    'use strict';

    // multi instance pattern
    function createInstance() {

        var app,
            win,
            container,
            view,
            considerSaved = false,
            conversation;

        app = ox.ui.createApp({
            name: 'com.voiceworks/ox-messenger/conversation/group',
            title: gt('Group conversation'),
            userContent: true,
            closable: true
        });

        app.create = function (user) {
            // set title, init model/view
            win.setTitle(gt('Create group conversation'));

            conversation = new GroupConversationModel({ user: Session.user });

            view = new CreateGroupConversationView({ model: conversation, app: this });

            view.on('save:start', function (e) {
                var convError = conversation.validate();
                if (convError) {
                    //view.showError(convError);
                    view.trigger('save:fail', convError);
                    return;
                }

                var roomId = Util.generateUUID(),
                    roomHost = settings.get('groupConversationRoomHost').toLowerCase(),
                    roomJid = roomId + '@' + roomHost,
                    roomName = conversation.get('roomName');

                conversation.set({
                    id: roomJid,
                    roomId: roomId,
                    roomHost: roomHost,
                    roomName: roomName,
                    timestamp: new Date().getTime(),
                    //#. "Last message" in newly created group conversation
                    lastMsg: new MessageModel({message: gt('Group started')}),
                    myRoom: true,
                    title: roomName,
                    sortName: roomName
                });

                win.busy();

                var participants = [];

                conversation.participants.each(function (participant) {
                    var user = Roster.getUserById(participant.get('email1'));
                    participants.push(user);
                });

                conversation.participants.reset();

                Aggregator.trigger('group:create', {
                    conversation: conversation,
                    participants: participants
                });

                // Add yourself
                //conversation.addParticipant(Session.user);

                Session.user.conversations.add(conversation);

                Aggregator.trigger('conversation:show', {
                    id: conversation.id
                });

                setTimeout(function () {
                    // sets the subject, has to be delayed since subject will only be displayed when the users are in the room
                    Aggregator.trigger('group:subject', { conversation: conversation });
                }, 500);
                view.trigger('save:success');
                //win.idle();
                //app.quit();
            });

            view.on('save:fail', function (event, data) {
                notifications.yell('error', gt('Failed to create group conversation. ') + _.escape(data));
                win.idle();
            });

            view.on('save:success', function () {
                notifications.yell('success', gt('Group conversation created'));
                considerSaved = true;
                win.idle();
                app.quit();
            });

            container.append(view.render().$el);
            container.addClass('io-ox-messenger');
            win.show();
        };

        app.setLauncher(function () {

            var state;
            win = ox.ui.createWindow({
                title: '',
                chromeless: true,
                name: 'com.voiceworks/ox-messenger/conversation/group'
            });
            app.setWindow(win);

            function fnToggleSave(isDirty) {
                app.getWindow().nodes.header.find('.btn[data-action="save"]').prop('disabled', !isDirty);
            }

            win.on('show', function () {
                if (!container.find('[data-extension-id="groupName"] input').val()) {
                    app.getWindow().nodes.header.find('.btn[data-action="save"]').prop('disabled', true);
                }
                container.find('input[type=text]:visible').eq(0).focus();
                container.find('[data-extension-id="groupName"] input').on('keyup', _.debounce(function () {
                    app.setTitle(_.noI18n($.trim($(this).val())) || gt('Group conversation'));
                    conversation.set('roomName', $(this).val());
                    fnToggleSave($(this).val());
                }, 150));
                container.find('[data-extension-id="groupDescription"] input').on('keyup', _.debounce(function () {
                    conversation.set('subject', $(this).val());
                }, 150));
            });

            container = $('<div>').addClass('container default-content-padding');

            win.nodes.main.addClass('scrollable').append(container);

            // hash state support
            state = app.getState();
            if ('id' in state) {
                app.edit(state);
            } else {
                app.create();
            }
        });

        app.setQuit(function () {
            var def = $.Deferred();

            def.resolve();

            return def;
        });

        return app;
    }

    return {

        getApp: createInstance,

        reuse: function (type, data) {
            if (type === 'edit') {
                return ox.ui.App.reuse('com.voiceworks/ox-messenger/conversation/group:edit.' + _.cid(data));
            }
        }
    };
});
