/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/views/call', [
    'io.ox/core/extensions',
    'io.ox/core/extPatterns/actions',
    'io.ox/core/notifications',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/core/session',
    'com.voiceworks/ox-messenger/core/views/disposable',
    'com.voiceworks/ox-messenger/core/views/timer',
    'com.voiceworks/ox-messenger/util',
    'com.voiceworks/ox-messenger/conversation/views/stream',
    'com.voiceworks/ox-messenger/core/views/partials/callpart',
    'gettext!com.voiceworks/ox-messenger',
    'com.voiceworks/ox-messenger/core/vendor/hoverIntent'
], function (ext, extPatternActions, CoreNotifications, Aggregator, Session, DisposableView, TimerView, Util,
             StreamView, CallPart, gt) {
    'use strict';

    /**
     * Call View
     */
    var CallView = DisposableView.extend({

        className: 'messenger--table__row  messenger--table__row--conversation-call  messenger--is-hidden',

        // Attributes
        controlHoverTimout: false,
        fullscreenActive: false,

        events: {
            'click .messenger--btn--call-action--fullscreen': 'toggleFullscreen',
            'click .messenger--btn--call-action--leave-fullscreen': 'toggleFullscreen',
            'click .messenger--btn--call-action--mute-video': 'toggleVideoMute',
            'click .messenger--btn--call-action--mute-audio': 'toggleAudioMute',
            'click .messenger--btn--call-action--hangup': 'hangup',
            'click .messenger--btn--call-action--group-call-invite': 'groupCallInvite'
        },

        initialize: function () {
            DisposableView.prototype.initialize.apply(this, arguments);

            this.listenTo(this.model.callParticipants, 'remove', this.removeCallStream);
            this.listenTo(this.model.callParticipants, 'add', this.addCallStream);
            this.listenTo(this.model.callParticipants, 'reset', this.resetCallStreams);

            this.listenTo(this.model, 'change:streamFrozen', this.handleFrozenUnFrozenStream);
        },

        show: function () {
            this.$el.removeClass('messenger--is-hidden');
        },

        hide: function () {
            this.$el.addClass('messenger--is-hidden');
        },

        showCallInfo: function (model) {
            var pictureUrl = model.getOtherParticipants().at(0).getImageUrl('large');

            this.$el.find('.messenger--call-info').removeClass('messenger--is-hidden');
            this.$el.find('.messenger--call-info .messenger--call-info__avatar').css({
                'background-image': 'url(' + pictureUrl + ')'
            });
            this.$el.find('.messenger--call-info .messenger--call-info__name').text(model.get('title'));
        },

        hideCallInfo: function () {
            this.$el.find('.messenger--call-info').addClass('messenger--is-hidden');
        },

        showCallStatus: function () {
            var state = Session.user.call.get('callState'),
                status = Session.user.call.getCallStateDisplayName();

            this.showCallConnecting(true);
            if (state === 'answered') {
                var timerView = new TimerView({
                    model: Session.user.call.timer
                });
                status = timerView.render().$el;
                this.showCallConnecting(false);
            }

            this.$el.find('.messenger--call-info .messenger--call-info__status').empty().append(status);
        },

        showVideo: function (media) {
            this.media = media;
            this.show();

            // There is a video saved, but 1-1 video
            if ($('#videoTmp').length && !this.model.isMuc()) {
                var tmpLocal = $('#videoLocal'),
                    tmpRemote = $('#videoRemote');

                if (tmpLocal.length) {
                    media.local = tmpLocal.get(0);
                }
                if (tmpRemote.length) {
                    media.remote = tmpRemote.get(0);
                }
            }

            if (!this.model.isMuc() && media.remote) {
                media.remote.setAttribute('height', '100%');
                media.remote.setAttribute('width', '100%');
                media.remote.setAttribute('autoplay', true);
                media.remote.setAttribute('id', 'videoRemote');

                this.progressListener = this.progressHandler.bind(this);
                this.timeupdateListener = this.timeupdateHandler.bind(this);
                $(media.remote)
                    .on('progress', this.progressListener)
                    .on('timeupdate', this.timeupdateListener);

                this.$el.find('.messenger--call-video').removeClass('messenger--is-hidden').append(media.remote);
            }

            if (media.local) {
                media.local.setAttribute('width', '200px');
                media.local.setAttribute('autoplay', true);
                media.local.setAttribute('id', 'videoLocal');
                this.$el.find('.messenger--call-mycam').removeClass('messenger--is-hidden').append(media.local);
            }

            setTimeout(function () {
                if ($('#videoTmp').length) {
                    $('#videoTmp').children().remove();
                    $('#videoTmp').remove();
                    if (media.local && media.local.play !== undefined) {
                        media.local.play();
                    }
                    if (media.remote && media.remote.play !== undefined) {
                        media.remote.play();
                    }
                }
            }, 100);
        },

        timeupdateHandler: function (event) {
            this.model.set('streamCurrentTime', new Date().getTime());
        },

        progressHandler: function (event) {
            var tmpTime        = new Date().getTime(),
                tmpFrozenLimit = 2000,
                frozen         = this.model.get('streamFrozen'),
                currentTime    = this.model.get('streamCurrentTime');

            if ((tmpTime - currentTime) > tmpFrozenLimit && !frozen && currentTime !== 0) {
                this.model.set('streamFrozen', true);
            }
            else if ((tmpTime - currentTime) <= tmpFrozenLimit && frozen && currentTime !== 0) {
                this.model.set('streamFrozen', false);
            }
        },

        handleFrozenUnFrozenStream: function (event) {
            var frozen = this.model.get('streamFrozen');

            if (frozen) {
                this.$el.find('.messenger--call-video__frozen').removeClass('messenger--is-hidden');
            } else {
                this.$el.find('.messenger--call-video__frozen').addClass('messenger--is-hidden');
            }
        },

        hideVideo: function () {
            this.hide();

            if (this.fullscreenActive) {
                BigScreen.exit();
            }

            this.$el.find('.messenger--call-video').addClass('messenger--is-hidden').children().remove();
            this.$el.find('.messenger--call-mycam').addClass('messenger--is-hidden').children().remove();

            this.removeProgressListener();
        },

        removeProgressListener: function () {
            if (this.media) {
                $(this.media.remote)
                    .off('progress', this.progressListener)
                    .off('timeupdate', this.timeupdateListener);
                this.model.unset('streamCurrentTime');
                delete this.media;
            }
        },

        showGroupCall: function (media) {
            var that = this;

            this.show();

            if (this.$el.find('.messenger--call-group-video__participant').length === 0) {
                this.model.callParticipants.each(function (participant) {
                    that.addCallStream(participant);
                });
            }

            // When Group Video Call
            if (Session.user.call.get('type') === 'groupvideo') {

                // @TODO why do we even save the videoLocal for groupCalls if we don't use it when switching?
                $('#videoLocal').remove();

                media.local.setAttribute('width', '200px');
                media.local.setAttribute('autoplay', true);
                media.local.setAttribute('id', 'videoLocal');

                if (media.local.paused) {
                    setTimeout(function () {
                        if (media.local.paused) {
                            media.local.play();
                        }
                    }, 100);
                }

                this.$el.find('.messenger--call-group-video').removeClass('messenger--is-hidden');
                this.$el.find('.messenger--call-mycam').removeClass('messenger--is-hidden').empty().append(media.local);

            // When Group Audio Call
            } else if (Session.user.call.get('type') === 'groupaudio') {
                // Merge user streams into one group stream.
                this.$el.find('.messenger--call-group-video').removeClass('messenger--is-hidden');
            }
        },

        addCallStream: function (participant) {
            if (!Session.user.call.get('type')){
                return;
            }

            // if (Session.user.call.get('type') === 'groupaudio') {
            //     if (!Util.isSameJid(participant.id, Session.user.id)){
            //         return;
            //     }
            // } else {
            //     if (Util.isSameJid(participant.id, Session.user.id)) {
            //         return;
            //     }
            // }

            // var streamView = new StreamView({
            //     model: participant,
            //     title: (Session.user.call.get('type') === 'groupaudio') ?
            //                 this.model.get('title') :
            //                 participant.getDisplayName()
            // });
            var streamView = new StreamView({
                model: participant,
                title: participant.getDisplayName()
            });

            var streamsElem = this.$el.find('.messenger--call-group-video__streams'),
                stream = streamView.render().$el;

            streamsElem.append(stream);
        },

        removeCallStream: function (participant) {
            this.$el.find('#stream-' + participant.id.replace(/[@|.]/g, '_')).remove();
        },

        resetCallStreams: function (data) {
            this.$el.find('.messenger--call-group-video__streams').children().remove();
        },

        hideGroupVideos: function () {
            this.hide();
            this.$el.find('.messenger--call-group-video').addClass('messenger--is-hidden').children().remove();
            this.$el.find('.messenger--call-mycam').addClass('messenger--is-hidden').children().remove();
        },

        showAudio: function (media) {
            this.show();

            media.remote.setAttribute('autoplay', true);
            media.remote.setAttribute('width', '1px');
            media.remote.setAttribute('height', '1px');
            media.remote.setAttribute('id', 'videoRemote');

            this.$el.find('.messenger--call-audio').append(media.remote).removeClass('messenger--is-hidden');

            if (media.remote && media.remote.play !== undefined) {
                media.remote.play();
            }
        },

        hideAudio: function () {
            this.hide();
            this.$el.removeClass('is-audio');
            this.$el.find('.messenger--call-audio').children().detach();
        },

        showControls: function (e) {
            clearTimeout(this.controlHoverTimeout);
            this.$el.find('.messenger--call-controls').addClass('is-active');

            // Hide buttons for audio call
            if (!Session.user.call.isCallSessionVideo()) {
                this.$el.find(
                    '.messenger--btn--call-action--mute-video, .messenger--btn--call-action--fullscreen'
                ).addClass('messenger--is-hidden');

            // Otherwise (video) show all controls
            } else {
                this.$el.find(
                    '.messenger--btn--call-action--mute-video, .messenger--btn--call-action--fullscreen'
                ).removeClass('messenger--is-hidden');
            }

            if (Session.user.call.isGroup()) {
                this.$el.find(
                    '.messenger--btn--call-action--group-call-invite'
                ).removeClass('messenger--is-hidden');
            }
        },

        hideControls: function (e) {
            var self = this;
            this.controlHoverTimeout = setTimeout(function () {
                self.$el.find('.messenger--call-controls').removeClass('is-active');
            }, 1000);
        },

        showCallConnecting: function (show) {
            if (show){
                this.$el.find('.messenger--throbber.call--connecting').removeClass('messenger--is-hidden');
            }
            else {
                this.$el.find('.messenger--throbber.call--connecting').addClass('messenger--is-hidden');
            }
        },

        toggleFullscreen: function () {
            var self = this,
                fullscreenContainer = this.$el.parents('#conversation-fullscreen-container');

            //@TODO add animation with an overlay for loading this view, currently the hiding of the elements is visible
            BigScreen.toggle(fullscreenContainer[0], function () {
                fullscreenContainer.addClass('is-fullscreen').find(
                    '.messenger--table__row--chat-messages, .messenger--table__row--chat-input'
                ).addClass('messenger--is-hidden');

                fullscreenContainer.find('.fa-expand').removeClass('fa-expand').addClass('fa-compress');

                fullscreenContainer.find('.messenger--table__cell--participants-list').addClass('messenger--is-hidden');

                self.fullscreenActive = true;
            }, function () {
                fullscreenContainer.removeClass('is-fullscreen').find(
                    '.messenger--table__row--chat-messages, .messenger--table__row--chat-input'
                ).removeClass('messenger--is-hidden');

                fullscreenContainer.find('.fa-compress').removeClass('fa-compress').addClass('fa-expand');

                fullscreenContainer.find('.messenger--table__cell--participants-list').removeClass('messenger--is-hidden');

                self.fullscreenActive = false;
            });
        },

        toggleVideoMute: function () {
            var muted = Session.user.call.get('callSession').toggleVideoMute();
            if (muted) {
                $('.messenger--btn--call-action--mute-video').addClass('active');
            } else {
                $('.messenger--btn--call-action--mute-video').removeClass('active');
            }
        },

        toggleAudioMute: function () {
            var muted = Session.user.call.get('callSession').toggleAudioMute();
            if (muted) {
                $('.messenger--btn--call-action--mute-audio').addClass('active');
            } else {
                $('.messenger--btn--call-action--mute-audio').removeClass('active');
            }
        },

        hangup: function () {
            Aggregator.trigger('call:hangup', {
                conversation: this.model
            });
        },

        groupCallInvite: function () {
            if (!this.model.canJoinGroupCall(Session.user.call.get('type'))) {
                CoreNotifications.yell('error', gt('It is not possible to add a new partcipant, you have reached the maximum number of group call participants.'));
                return;
            }

            var baton = new ext.Baton({
                data: this.model,
                isGroupCallInvite: true
            });

            if (this.fullscreenActive) {
                BigScreen.exit();
            }

            extPatternActions.invoke('com.voiceworks/ox-messenger/actions/addparticipant', this, baton);
        },

        render: function () {
            this.$el.children().remove();
            this.$el.prevObject = null;

            this.$el.append(
                $('<div class="messenger--table__cell  messenger--table__cell--conversation-call">').append(
                    $('<div class="messenger--table">').append(
                        $('<div class="messenger--table__row  messenger--table__row--call">').append(
                            $('<div class="messenger--call">').append(
                                $('<div class="messenger--call-video__frozen  messenger--is-hidden">').append(
                                    $('<i class="fa fa-spinner fa-spin">')
                                ),
                                $('<div class="messenger--call-audio  messenger--is-hidden">'),
                                $('<div class="messenger--call-video  messenger--is-hidden">'),
                                $('<div class="messenger--call-info  messenger--is-hidden">').append(
                                    $('<div class="messenger--avatar messenger--avatar--large  messenger--call-info__avatar"></div>'),
                                    $('<div class="messenger--call-info__line  messenger--call-info__name  messenger--h2  messenger--font--light"></div>'),
                                    $('<div class="messenger--call-info__line  messenger--call-info__status  messenger--h4  messenger--font--light"></div>'),
                                    CallPart.getConnectingAnimation()
                                ),
                                $('<div class="messenger--call-group-video  messenger--is-hidden">').append(
                                    $('<div class="messenger--table  messenger--call-group-video__streams">')
                                )
                            )
                        )
                    ),
                    $('<div class="messenger--call-mycam  messenger--is-hidden">'),
                    $('<div class="messenger--call-controls">').append(
                        $('<button class="btn messenger--btn--call-action messenger--btn--call-action--group-call-invite  messenger--is-hidden" data-action="group-call-invite" tabindex="2" type="button">').attr({
                'title': gt('Invite user to group call')
            }).append(
                            $('<i class="fa fa-plus  messenger--fa-plus--large"></i>')
                        ),
                        $('<button class="btn messenger--btn--call-action messenger--btn--call-action--mute-video">').attr({
                'title': gt('Disable camera'),
                'data-action': 'video',
                'tabindex': '2',
                'type': 'button'
            }).append(
                            $('<i class="fa fa-eye-slash"></i>')
                        ),
                        $('<button class="btn messenger--btn--call-action messenger--btn--call-action--hangup">').attr({
                'title': gt('Hangup call'),
                'data-action': 'hangup',
                'tabindex': '3',
                'type': 'button'
            }).append(
                            $('<i class="fa fa-phone"></i>')
                        ),
                        $('<button class="btn messenger--btn--call-action messenger--btn--call-action--mute-audio">').attr({
                'title': gt('Mute microphone'),
                'data-action': 'mute',
                'tabindex': '4',
                'type': 'button'
            }).append(
                            $('<i class="fa fa-microphone-slash"></i>')
                        ),
                        $('<button class="btn messenger--btn--call-action messenger--btn--call-action--fullscreen">').attr({
                'title': gt('View in full screen'),
                'data-action': 'fullscreen',
                'tabindex': '5',
                'type': 'button'
            }).append(
                            $('<i class="fa fa-expand"></i>')
                        )
                    )
                )
            );

            this.$el.hoverIntent(
                $.proxy(this.showControls, this),
                $.proxy(this.hideControls, this)
            );

            return this;
        }

    });

    return CallView;
});
