/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/views/conversation', [
    'io.ox/core/notifications',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/conversation/models/conversation',
    'com.voiceworks/ox-messenger/conversation/models/message/factory',
    'com.voiceworks/ox-messenger/core/views/disposable',
    'com.voiceworks/ox-messenger/conversation/views/conversationheader',
    'com.voiceworks/ox-messenger/conversation/views/conversationparticipantslist',
    'com.voiceworks/ox-messenger/conversation/views/call',
    'com.voiceworks/ox-messenger/conversation/views/messagecontainercollection',
    'com.voiceworks/ox-messenger/conversation/views/messageform',
    'com.voiceworks/ox-messenger/file/views/dropzone',
    'com.voiceworks/ox-messenger/file/main',
    'com.voiceworks/ox-messenger/util',
    'gettext!com.voiceworks/ox-messenger',
    'less!com.voiceworks/ox-messenger/conversation/style'
], function (notifications, Aggregator, ConversationModel, MessageFactory, DisposableView, ConversationHeaderView, ConversationParticipantsList,
            CallView, MessageContainerCollectionView, MessageFormView, DropzoneView, FileController, Util, gt) {
    'use strict';

    /**
     * Conversation View
     */
    var ConversationView = DisposableView.extend({
        id: 'conversation-fullscreen-container',
        className: 'messenger--table  messenger--table--conversation',

        initialize: function (options) {
            DisposableView.prototype.initialize.apply(this, arguments);

            this.Session = require('com.voiceworks/ox-messenger/core/session');
            this.user    = options.user;
            this.model   = this.model || new ConversationModel();

            this.listenTo(this.Session.user.call, 'change:callState', this.stateCheck);
            this.listenTo(this.model.messageContainers, 'reset', this.render);
            this.listenTo(this.model.messageContainers, 'add', this.scroll);
            this.listenTo(this.model, 'change:scroll', this.scroll);

            // File upload events
            this.listenTo(Aggregator, 'file:uploadstart', $.proxy(this.onFileUploadStart, this));
            this.listenTo(Aggregator, 'file:uploadend', $.proxy(this.onFileUploadEnd, this));
            this.listenTo(Aggregator, 'file:picked', $.proxy(this.onFilePicked, this));
            this.listenTo(Aggregator, 'file:uploadcancelled', $.proxy(this.onFileUploadCancelled, this));
            this.listenTo(Aggregator, 'file:permisiondenied', $.proxy(this.onFilePermissionDenied, this));
            this.listenTo(Aggregator, 'file:permisionapproved', $.proxy(this.onFilePermissionApproved, this));
        },

        // --------------------------------------- View Methods -------------------------------

        render: function () {
            var conversation = this.model;

            this.$el.children().remove();
            this.$el.prevObject = null;

            // Conversation Header
            this.conversationHeaderView = new ConversationHeaderView({
                user: this.user,
                model: conversation
            });

            var conversationParticipantsList = null;
            if (this.model.isMuc()) {
                // Conversation Participants List
                this.conversationParticipantsList = new ConversationParticipantsList({
                    model: conversation,
                    conversationView: this
                });
                conversationParticipantsList = this.conversationParticipantsList.render().$el;
            }

            // Message Collection
            this.messageContainerCollectionView = new MessageContainerCollectionView({
                collection: conversation.messageContainers
            });

            // Message Form
            this.messageFormView = new MessageFormView({
                user: this.user,
                model: conversation
            });

            // Call View
            // TODO: render callview only when we actually need it
            this.callView = new CallView({
                model: conversation
            });

            // Dropzone
            this.dropzoneView = new DropzoneView({
                caption: gt('Drop files here to upload'),
                model: conversation
            });

            this.$el.append(
                $('<div class="messenger--table__cell">').append(
                    $('<div class="messenger--table">').append(
                        this.conversationHeaderView.render().$el,
                        $('<div class="messenger--table__row">').append(
                            $('<div class="messenger--table__cell">').append(
                                $('<div class="messenger--table">').append(
                                    this.callView.render().$el,
                                    this.messageContainerCollectionView.render().$el.append(
                                        this.dropzoneView.render().$el
                                    ),
                                    this.messageFormView.render().$el
                                )
                            )
                        )
                    )
                ),
                conversationParticipantsList
            );

            this.stateCheck();

            return this;
        },

        // This function only rerender messages container
        renderMessages: function(){
            this.messageContainerCollectionView.$el.replaceWith(this.messageContainerCollectionView.render().$el.append(
                this.dropzoneView.render().$el
            ));
        },

        scroll: function (force) {
            if (this.messageContainerCollectionView) {
                this.messageContainerCollectionView.scroll(force);
                this.model.set('scroll', false);
            }
        },

        setMessagingHeight: function () {
            // TODO remove browser hack
            var isIE10 = (navigator.appVersion.indexOf('MSIE 10') !== -1),
                isIE11 = (navigator.appVersion.indexOf('Trident/7') !== -1);

            if (!isIE10 && !isIE11) {
                return;
            }

            // calc right window
            var heightRightWindow = $('.messenger--window-right-container').height();
            // calc header
            var heightHeader = $('.messenger--table__row--conversation-header').height();
            // calc input
            var heightInput = $('.messenger--table__row--chat-input').height();
            // height available
            var heightAvailable = (heightRightWindow - heightHeader - heightInput);

            // var callWindow = $('.messenger--call');
            var callWindow = $('.messenger--table__row--conversation-call');
            if (callWindow.length && !callWindow.hasClass('messenger--is-hidden')) {
                heightAvailable = heightAvailable / 2;
            }

            // Set height for participants list
            $('.messenger--participants-list').height(heightRightWindow);

            // messages wrapper element
            var messagesWrapper = $('.messenger--chat-messages-wrapper');
            messagesWrapper.height(heightAvailable);
        },

        // ------------------------------------- Call States ---------------------------------

        stateCheck: function () {
            if (this.Session.user.call.get('type') === 'pstn') {
                return;
            }

            if (this.Session.user.call.get('conversationId') !== this.model.id) {
                return;
            }

            switch (this.Session.user.call.get('callState')) {
            case 'group-initiated':
                this.stateIsGroupInit();
                break;
            case 'trying':
            case 'in-progress':
                this.stateIsInProgress();
                break;
            case 'answered':
                this.stateIsAnswered();
                break;
            case 'idle':
                this.stateIsIdle();
                break;
            }
        },

        // Group init => Show your own video and avatars of the invited users
        stateIsGroupInit: function () {
            if (this.model.get('initiator') === this.Session.user && this.model.getParticipantsActiveCallCount() < 2) {
                Util.playSound('dialtone.mp3', 'audioDial', true);
            }

            this.conversationHeaderView.hide();
            this.callView.show();
            this.callView.showGroupCall(this.Session.user.call.get('callSession').getMediaCanvas());
            this.messageContainerCollectionView.scroll();
        },

        // In Progress => Call is dialing or you're being called
        stateIsInProgress: function () {
            if (this.model && this.model.isMuc()) {
                return;
            }

            Util.playSound('dialtone.mp3', 'audioDial', true);
            this.conversationHeaderView.hide();
            this.callView.show();
            this.callView.showCallInfo(this.model);
            this.callView.showCallStatus();
            this.setMessagingHeight();
            this.messageContainerCollectionView.scroll();
        },

        // Answered => Call is answered and running
        stateIsAnswered: function () {
            Util.stopAllSounds();

            this.conversationHeaderView.hide();

            // Hide CallInfo for video call
            if (this.Session.user.call.isCallSessionVideo()) {
                this.callView.hideCallInfo();
                if ($('#videoTmp').length) {
                    this.callView.showVideo({local: '', remote: ''});
                } else {
                    this.callView.showVideo(this.Session.user.call.get('callSession').getMediaCanvas());
                }

            // Otherwise (audio) show CallInfo and CallStatus
            } else {
                this.callView.showCallInfo(this.model);
                this.callView.showCallStatus();
                this.callView.showAudio(this.Session.user.call.get('callSession').getMediaCanvas());
            }

            this.setMessagingHeight();
            this.messageContainerCollectionView.scroll();
        },

        // Idle => Call is idle and has stopped
        stateIsIdle: function (model) {
            if (this.Session.user.call.isActive() && this.Session.user.call.get('callSession').status === 'failed') {
                Util.playSound('busy.mp3', 'audioBusy', false);
            } else {
                Util.stopAllSounds();
            }

            this.callView.hideCallInfo();
            this.callView.hideVideo();
            this.callView.hideAudio();
            this.conversationHeaderView.show();
            this.setMessagingHeight();
            this.messageContainerCollectionView.scroll();
            this.model.set('lastCallFrozen', this.model.get('streamFrozen'));
            this.model.set('streamFrozen', false);

            $('.messenger--btn--call-action--mute-video').removeClass('active');
            $('.messenger--btn--call-action--mute-audio').removeClass('active');
        },

        // ------------------------------------------- File Methods ----------------------------------------

        completeFileMessage: function(message) {
            var that = this,
                conversation = this.model,
                fileDesc = message.get('fileDesc');

            // Check the file upload was not cancelled by the user
            if(!fileDesc.get('upload_canceled')){
                message.set('message', fileDesc.get('filename'));
                FileController.shareFile(fileDesc, conversation.participants)
                    .done(function(fileDesc){
                        that.renderMessages();
                        that.scroll(true);
                        // Notify participants about the file upload.
                        // It will create the persistent XMPP message
                        Aggregator.trigger('message:send', message);
                    })
                    .fail(function () {
                        fileDesc.set('upload_canceled', true);
                        // Notify the app the upload was cancelled
                        Aggregator.trigger('file:uploadcancelled', fileDesc);
                    });
            }
        },

        // ------------------------------------------- File Upload Callbacks ----------------------------------

        onFileUploadStart: function (fileDesc) {
            var conversation = this.model,
                message = MessageFactory.create({
                    username     : this.Session.user.id,
                    from         : this.Session.user.id,
                    message      : gt('Uploading file...'),
                    timestamp    : new Date().getTime(),
                    isHistory    : false,
                    conversation : conversation,
                    fileDesc     : fileDesc
                });

            // Add message to the conversation, without notifying participants yet
            conversation.addMessage(message);

            return message;
        },

        onFileUploadEnd: function (fileDesc) {
            var conversation = this.model,
                message = conversation.getMessageByFileDesc(fileDesc);

            this.completeFileMessage(message);
        },

        onFilePicked: function (fileDesc){
            var message = this.onFileUploadStart(fileDesc);

            this.completeFileMessage(message);
        },

        onFileUploadCancelled: function() {
            this.renderMessages();
            this.scroll(true);
        },

        onFilePermissionDenied: function() {
            this.renderMessages();
            this.scroll(true);
        },

        onFilePermissionApproved: function() {
            this.renderMessages();
            this.scroll(true);
        }

    });

    return ConversationView;
});
