/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/views/conversationparticipantslist', [
    'io.ox/core/extensions',
    'com.voiceworks/ox-messenger/conversation/models/conversation',
    'com.voiceworks/ox-messenger/roster/views/userpresence',
    'com.voiceworks/ox-messenger/core/session',
    'com.voiceworks/ox-messenger/core/views/disposable',
    'gettext!com.voiceworks/ox-messenger'
], function (ext, ConversationModel, UserPresenceView, Session, DisposableView, gt) {
    'use strict';

    /**
     * Conversation Header View
     */
    var ConversationParticipantsList = DisposableView.extend({
        className: 'messenger--table__cell  messenger--table__cell--participants-list',

        initialize: function (options) {
            DisposableView.prototype.initialize.apply(this, arguments);

            _.bindAll(this, 'render', 'beforeRender', 'afterRender');
            this.render = _.wrap(this.render, function (render) {
                this.beforeRender();
                render();
                this.afterRender();
                return this;
            });

            this.model = this.model || new ConversationModel();
            this.listenTo(this.model.participants, 'add remove reset', this.render);
            this.listenTo(Session.user, 'change:status', this.render);
            this.conversationView = options.conversationView;
        },

        render: function () {
            var that = this,
                conversation = this.model,
                $container = $('<div class="messenger--participants-list">');

            this.$el.children().remove();

            $container.append(
                //#. Conversation participants header
                $('<div class="messenger--h3  messenger--font--light">').text(gt('Participants'))
            );

            var $list = $('<ul class="messenger--vertical-list  messenger--vertical-list__participants-list">');

            conversation.participants.each(function (participant) {
                // Removed due to request to show offline users
                // if (participant.get('isInvisible')) {
                //     return;
                // }

                var participantView = new ConversationParticipantView({
                    model: participant,
                    containerClass: '',
                    haloLink: true,
                    roomId: conversation.get('roomId')
                });

                //that.addChildView(participantView);

                var phoneIcon = (participant.get('inGroupCall') === conversation.get('roomId')) ? $('<i class="fa fa-phone  fa-phone--participant-list">') : '';

                $list.append(
                    $('<li class="messenger--text-wrap  messenger--text-color--theme-sec  messenger--h5">').append(
                        participantView.render().$el
                    )
                );
            });

            $container.append($list);

            this.$el.append($container);

            return this;
        },

        beforeRender: function () {

        },

        afterRender: function () {
            if (this.conversationView) {
                this.conversationView.setMessagingHeight();
            }
        }
    });

    var ConversationParticipantView = DisposableView.extend({
        initialize: function (options) {
            DisposableView.prototype.initialize.apply(this, arguments);
            this.listenTo(this.model, 'change:inGroupCall', this.render);
            this.listenTo(this.model, 'change:isSpeaking', this.render);

            this.options = options || {};
        },
        render: function () {
            this.$el.empty();

            var userPresenceView = new UserPresenceView({
                model: this.model,
                containerClass: '',
                haloLink: this.options.haloLink
            });

            var phoneIcon = '';
            var speakingIcon = '';
            if (this.model.get('inGroupCall') && this.model.get('inGroupCall') === this.options.roomId) {
                phoneIcon = $('<i class="fa fa-phone  fa-phone--participant-list">');
                if (this.model.get('isSpeaking')) {
                    speakingIcon = $('<i class="fa fa-volume-up fa-volume-up--participant-list">');
                }
            }

            this.$el.append(
                $('<li class="messenger--text-wrap  messenger--text-color--theme-sec  messenger--h5">').append(
                    phoneIcon,
                    speakingIcon,
                    userPresenceView.render().$el
                )
            );

            return this;
        }
    });

    return ConversationParticipantsList;
});
