/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/views/messagecontainer_chat', [
    'io.ox/core/extensions',
    'com.voiceworks/ox-messenger/core/session',
    'com.voiceworks/ox-messenger/conversation/views/message_chat',
    'com.voiceworks/ox-messenger/conversation/views/messagecontainer',
    'com.voiceworks/ox-messenger/core/vendor/virtual-dom',
    'com.voiceworks/ox-messenger/util',
    'gettext!com.voiceworks/ox-messenger'
], function (ext, Session, ChatMessageView, MessageContainerView, vdom, Util, gt) {
    'use strict';

    var h = vdom.h;

    /**
     * Chat Message Container View
     * this.model = message container
     * this.model.conversation = conversation
     * this.model.messages = messages collection
     * this.model.senderUser = sender User
     */
    var ChatMessageContainerView = MessageContainerView.extend({

        initialize: function (options) {
            this.parent = options.parent;
        },

        template: function () {
            var senderContainer = '';
            var imgUrl = this.model.senderUser.getImageUrl('small');
            var isMyself = Util.isSameJid(this.model.senderUser.id, Session.user.id);

            var pictureContainer = h('div.messenger--media__img.messenger--avatar' +
                '.messenger--avatar--small.messenger--avatar--message', {
                    attributes: {
                        jid: this.model.senderUser.id
                    },
                    style: {
                        'background-image': 'url("' + imgUrl + '")'
                    }
                }
            );

            if (!isMyself) {
                senderContainer = h('div.messenger--media__sender.messenger--h6' +
                    '.messenger--font--bold.messenger--text-color--dark-gray',
                    this.model.senderUser.getDisplayName()
                );
            }

            var rootClass = '';
            if (isMyself) {
                rootClass = '.messenger--media--chat-message--myself';
            }

            var children = this.model.messages.map(function (message) {
                var messageView = new ChatMessageView({
                    model: message,
                    conversation: this.model.conversation
                });

                return messageView.template();
            }, this);

            return h('div.messenger--media.messenger--media--chat-message' +
                    rootClass, [
                senderContainer,
                pictureContainer,
                h('div.messenger--media__body', children)
            ]);
        }
    });

    return ChatMessageContainerView;
});
