define('com.voiceworks/ox-messenger/core/heap-counter', [], function () {

    /**
     * Requires chrome to be started with the --enable-precise-memory-info flag.
     *
     * @param container
     * @constructor
     */
    function HeapCounter(container) {

        this.element = document.createElement('div');
        this.element.classList.add('heap-counter', 'messenger--font--bold', 'messenger--text--center');

        if (container) {
            this.render(container);
        }
    }

    HeapCounter.prototype.element = null;
    HeapCounter.prototype.timeoutId = null;

    HeapCounter.isAvailable = function () {
        return !!(window.performance && window.performance.memory);
    };

    HeapCounter.prototype.render = function (container) {
        if (!container && !this.element.parentNode) {
            throw new Error('Container is missing');
        }

        if (!this.element.parentNode) {
            container.appendChild(this.element);
        }

        var unit = Math.pow(1024, 2);

        var memory = window.performance.memory;
        var allocated = memory.usedJSHeapSize / unit;
        allocated = Math.round(allocated * 100) / 100;
        var total = memory.totalJSHeapSize / unit;
        total = Math.round(total * 100) / 100;

        this.element.innerHTML = allocated + ' MiB / ' + total + ' MiB';
        this.timeoutId = setTimeout(this.render.bind(this), 500);
    };

    HeapCounter.prototype.dispose = function () {
        if (this.element.parentNode) {
            this.element.parentNode.removeChild(this.element);
        }
        if (this.timeoutId) {
            clearTimeout(this.timeoutId);
        }
    };

    return HeapCounter;
});
