/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/core/notification', [
    'com.voiceworks/ox-messenger/util',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/roster/main',
    'com.voiceworks/ox-messenger/core/notification-control',
    'settings!com.voiceworks/ox-messenger',
    'gettext!com.voiceworks/ox-messenger'
], function (Util, Aggregator, Roster, NotificationControl, Settings, gt) {
    'use strict';

    var NotificationController = function () { };

    NotificationController.prototype = {
        warn: (function () {
            window.ignoreTimer = null;
            clearTimeout(window.ignoreTimer);

            var validType = /^(busy|error|info|success|warning|screenreader)$/,
                active = false,
                timer = null,
                isSmartphone = _.device('smartphone'),
                clickCallback,

                durations = {
                    busy: 10000,
                    error: 30000,
                    info: 10000,
                    success: 4000,
                    warning: 10000,
                    screenreader: 100,
                },

                icons = {
                    busy: 'fa fa-refresh fa-spin',
                    error: 'fa fa-exclamation',
                    info: 'fa fa-exclamation',
                    success: 'fa fa-check',
                    warning: 'fa fa-exclamation'
                },

                remove = function (immediately) {

                    active = false;
                    clearTimeout(timer);

                    if (immediately) {
                        $('.io-ox-alert').remove();
                        return;
                    }

                    var nodes = $('.io-ox-alert')
                        .trigger('notification:removed')
                        .removeClass('appear');

                    // has been event-based (transitionend webkitTransitionEnd) but sometimes
                    // such events are not triggered causing invisible but blocking overlays
                    setTimeout(function () {
                        nodes.remove();
                        nodes = null;
                    }, 300);
                },

                click = function (e) {
                    if (!active) {
                        return;
                    }

                    var target = $(e.target),
                        alert = target.closest('.io-ox-alert');

                    if (!$(e.target).closest('.io-ox-alert').length) {
                        return remove();
                    }

                    if (target.hasClass('close') || target.hasClass('fa-times')) {
                        return remove();
                    } else {
                        if (clickCallback) {
                            clickCallback();
                        }
                    }
                    return remove();
                };

            $(document).on(_.device('touch') ? 'tap' : 'click', click);

            return function (type, message, callbackFn) {

                var o = {
                    duration: 0,
                    html: false,
                    type: 'info'
                };

                clickCallback = callbackFn;

                o.type = type || 'info';
                o.message = message;

                active = false;

                clearTimeout(timer);
                timer = o.duration === -1 ? null : setTimeout(remove, o.duration || durations[o.type] || 5000);

                var html = o.html ? o.message : _.escape(o.message).replace(/\n/g, '<br>'),
                    reuse = false,
                    className = 'io-ox-alert io-ox-alert-info ox-messenger',
                    wordbreak = 'normal';

                // reuse existing alert?
                var node = $('.io-ox-alert');

                if (node.length) {
                    //screenreader should not reuse existing notifications, this would only remove them for other users
                    node.empty();
                    reuse = true;
                    className += 'appear';
                } else {
                    node = $('<div role="alert" tabindex="-1">');
                }

                node.attr('class', className).append(
                    $('<div class="icon">').append(
                        $('<i>').attr('aria-hidden', true).addClass(icons[o.type] || 'fa fa-fw')
                    )
                );

                node.append(
                    $('<div role="alert" aria-live="polite" class="message user-select-text">').append(
                        o.headline ? $('<h2 class="headline">').text(o.headline) : [],
                        $('<div>').css('word-break', wordbreak).html(html)
                    )
                );

                node.append(
                    $('<a href="#" role="button" class="close" tabindex="1">').append(
                        $('<i class="fa fa-times" aria-hidden="true">'),
                        $('<span class="sr-only">').text(gt('Click to close this notification')))
                );

                if (!reuse) {
                    $('#io-ox-core').append(node);
                }

                // put at end of stack not to run into opening click
                setTimeout(function () {
                    active = true;
                    if (!reuse) {
                        node.addClass('appear');
                    }
                }, _.device('touch') ? 300 : 0);
            };
        }()),
        yell: (function () {

            //$('#io-ox-core').prepend($('<div id="io-ox-notifications-popups">'));

            var validType = /^(busy|error|info|success|warning|screenreader)$/,
                active = false,
                timer = null,
                isSmartphone = _.device('smartphone'),

                durations = {
                    busy: 10000,
                    error: 30000,
                    info: 10000,
                    success: 4000,
                    warning: 10000,
                    screenreader: 100,
                },

                icons = {
                    busy: 'fa fa-refresh fa-spin',
                    error: 'fa fa-exclamation',
                    info: 'fa fa-exclamation',
                    success: 'fa fa-check',
                    warning: 'fa fa-exclamation'
                },

                remove = function (immediately) {

                    active = false;
                    clearTimeout(timer);

                    if (immediately) {
                        $('.io-ox-alert').remove();
                        return;
                    }

                    var nodes = $('.io-ox-alert')
                        .trigger('notification:removed')
                        .removeClass('appear');

                    // has been event-based (transitionend webkitTransitionEnd) but sometimes
                    // such events are not triggered causing invisible but blocking overlays
                    setTimeout(function () {
                        nodes.remove();
                        nodes = null;
                    }, 300);
                },

                click = function (e) {
                    if (!active) {
                        return;
                    }

                    var target = $(e.target),
                        alert = target.closest('.io-ox-alert');

                    if (!$(e.target).closest('.io-ox-alert').length) {
                        return remove();
                    }

                    if ($(e.target).hasClass('close')) {
                        return remove();
                    } else if (target.data('jid')) {
                        Aggregator.trigger('conversation:show', {id: target.data('jid'), type: 'chat'});
                        return remove();
                    }
                };

            $(document).on(_.device('touch') ? 'tap' : 'click', click);

            return function (user, message) {

                var o = {
                    duration: 0,
                    html: false,
                    type: 'info'
                };

                o.message = message.get('message').substr(0, 140);

                active = false;

                clearTimeout(timer);
                timer = o.duration === -1 ? null : setTimeout(remove, o.duration || durations[o.type] || 5000);

                var html = o.html ? o.message : _.escape(o.message).replace(/\n/g, '<br>'),
                    reuse = false,
                    className = 'io-ox-alert io-ox-alert-info ox-messenger',
                    wordbreak = html.indexOf('http') >= 0 ? 'break-all' : 'normal';

                // reuse existing alert?
                var node = $('.io-ox-alert');

                if (node.length) {
                    //screenreader should not reuse existing notifications, this would only remove them for other users
                    node.empty();
                    reuse = true;
                    className += 'appear';
                } else {
                    node = $('<div role="alert" tabindex="-1">');
                }

                var jid = '';

                if (message.get('muc')) {
                    jid = message.get('roomId') + '@' + message.get('host');
                } else {
                    jid = user.id;
                }

                node.addClass(className).append(
                    $('<div class="messenger--media  messenger--media--notifier-alert">').append(
                        $('<div class="messenger--media__img  messenger--media__img-avatar  messenger--avatar  messenger--avatar--small">').css({
                    'background-image': 'url(' + user.getImageUrl('small') + ')'
                }),
                        $('<div class="messenger--media__body  messenger--media__body--notifier-alert">').append(
                            $('<div class="messenger--text-wrap">').append(
                                //#. Conversation message notification. Example: John Smith says: Hello
                                gt('%s says: %s', gt.noI18n('<span class="messenger--font--bold">' + _.escape(user.getDisplayName()) + '</span>'), html)
                            ).data('jid', jid)
                        )
                    ),
                    $('<a href="#" class="close" tabindex="1">').html('&times')
                );

                if (!reuse) {
                    $('#io-ox-core').append(node);
                }

                // put at end of stack not to run into opening click
                setTimeout(function () {
                    active = true;
                    if (!reuse) {
                        node.addClass('appear');
                    }
                }, _.device('touch') ? 300 : 0);
            };
        }()),
        call: (function () {

            var active = false,
                timer = null,
                isSmartphone = _.device('smartphone'),

                remove = function (immediately) {

                    active = false;
                    clearTimeout(timer);

                    if (immediately) {
                        $('.io-ox-alert').remove();
                        return;
                    }

                    $('.io-ox-alert')
                        .trigger('notification:removed')
                        .on('transitionend webkitTransitionEnd', function () {
                            $(this).remove();
                        })
                        .removeClass('slide-in');
                },

                click = function (e) {
                    if (!active) {
                        return;
                    }

                    if (isSmartphone) {
                        return remove();
                    }

                    var target = $(e.target), alert = target.closest('.io-ox-alert');

                    // click on notification?
                    if (alert.length) {
                        // don't close on plain links
                        if (target.is('a') && !target.hasClass('close')) {
                            return;
                        }
                        // close if clicked on close icon or if clicked on success notifications
                        if (target.hasClass('close') || alert.hasClass('io-ox-alert-success')) {
                            e.preventDefault();
                            remove();
                        }
                    } else {
                        remove();
                    }
                };

            $(document).on('click tap', click);

            return {
                create: function (user, type, callSession, conversation) {

                    NotificationControl.notify(conversation, {
                        sound: function () {
                            Util.playSound(Settings.get('incomingCallRingtone'), 'audioRingtone', true, false, true);
                        }
                    });

                    var o = {
                        duration: Settings.get('groupCallAnswerTimeout') * 1000,
                        html: false
                    };

                    var self = this;
                    active = false;
                    clearTimeout(timer);

                    // After timeout, remove notification and send decline to other participants
                    timer = setTimeout(function () {
                        self.destroy();
                        if (type === 'groupvideo' || type === 'groupaudio') {
                            Aggregator.trigger('groupcall:decline', {
                                conversation: conversation,
                                type: (type === 'video' || type === 'groupvideo') ? 'video' : 'audio',
                                caller: user,
                                session: callSession
                            });
                        }
                    }, Settings.get('groupCallAnswerTimeout') * 1000);

                    var reuse = false,
                        node = $('<div>').addClass('messenger--notifier-container'),
                        activeWindow = $('#' + ox.ui.App.getCurrentApp().get('window').id);

                    Aggregator.trigger('roster:hide', {});
                    Aggregator.trigger('phone:hide', {});

                    //$('#io-ox-notifications-overlay').trigger('click');
                    require('io.ox/core/notifications').hide();

                    node.attr('data-uri', callSession.sip_uri);

                    var typetxt = '';
                    switch (type) {
                    case 'video':
                        typetxt = gt('%s is video calling you...', user.getDisplayName());
                        break;
                    case 'audio':
                        typetxt = gt('%s is calling you...', user.getDisplayName());
                        break;
                    case 'groupvideo':
                        typetxt = gt('%s is group video calling you...', user.getDisplayName());
                        break;
                    case 'groupaudio':
                        typetxt = gt('%s is group calling you...', user.getDisplayName());
                        break;
                    }

                    node.append(
                        $('<div class="messenger--notifier">').append(
                            $('<div class="messenger--notifier__user">').append(
                                $('<div class="messenger--media">').append(
                                    $('<div class="messenger--media__img  messenger--media__img-avatar  messenger--avatar  messenger--avatar--large">').css({
                        'background-image': 'url(' + user.getImageUrl() + ')'
                    }),
                                    $('<div class="messenger--media__body  messenger--media__body--notifier">').append(
                                        $('<div class="messenger--h2  messenger--text-wrap  messenger--font--light">').text(typetxt)
                                    )
                                )
                            ),
                            $('<div class="messenger--notifier__status  messenger--h4  messenger--font--light">').append(
                                $('<div class="throbber  messenger--throbber  messenger--throbber--large">').append(
                                    $('<div class="bounce1">'),
                                    $('<div class="bounce2">'),
                                    $('<div class="bounce3">')
                                )
                            ),
                            $('<div class="messenger--notifier__actions">').append(
                                $('<button type="button" class="btn btn-primary messenger--btn--accept  messenger--btn--notifier__action">').text(gt('Accept')).click(function (event) {
                                    event.preventDefault();
                                    if (type === 'groupvideo' || type === 'groupaudio') {
                                        Aggregator.trigger('groupcall:accept', {
                                            conversation: conversation,
                                            type: (type === 'groupvideo') ? 'video' : 'audio',
                                            caller: user
                                        });
                                    } else {
                                        Aggregator.trigger('call:accept', {
                                            contact: user.get('contact'),
                                            session: callSession
                                        });
                                    }
                                    self.destroy();
                                }),
                                $('<button type="button" class="btn btn-primary messenger--btn--decline  messenger--btn--notifier__action">').text(gt('Decline')).click(function (event) {
                                    event.preventDefault();
                                    if (type === 'groupvideo' || type === 'groupaudio') {
                                        Aggregator.trigger('groupcall:decline', {
                                            conversation: conversation,
                                            type: (type === 'groupvideo') ? 'video' : 'audio',
                                            caller: user,
                                            session: callSession
                                        });
                                    } else {
                                        Aggregator.trigger('call:decline', {
                                            contact: user.get('contact'),
                                            session: callSession
                                        });
                                    }
                                    self.destroy();
                                }),
                                $('<button type="button" class="btn btn-primary messenger--btn--ignore  messenger--btn--notifier__action">').text(gt('Ignore')).click(function (event) {
                                    event.preventDefault();

                                    // After timeout, send decline to other participants
                                    window.ignoreTimer = setTimeout(function () {
                                        if (type === 'groupvideo' || type === 'groupaudio') {
                                            Aggregator.trigger('groupcall:decline', {
                                                conversation: conversation,
                                                type: (type === 'groupvideo') ? 'video' : 'audio',
                                                caller: user,
                                                session: callSession
                                            });
                                        } else {
                                            Aggregator.trigger('call:decline', {
                                                contact: user.get('contact'),
                                                session: callSession
                                            });
                                        }
                                    }, Settings.get('groupCallAnswerTimeout') * 1000);

                                    self.destroy();
                                })
                            )
                        )
                    );

                    //$('.io-ox-alert').remove();
                    // node.addClass('slide-in'); /
                    reuse = false;
                    //if (!reuse) $('.window-body', activeWindow).prepend(node);
                    if (!reuse) {
                        $('#io-ox-screens').prepend(node);
                    }

                    // put at end of stack not to run into opening click
                    setTimeout(function () {
                        active = true;
                        if (!reuse) {
                            node.addClass('slide-in');
                        }
                    }, 300);
                },
                destroy: function () {
                    $('.messenger--notifier-container').removeClass('slide-in');
                    Util.stopAllSounds();

                    setTimeout(function () {
                        $('.messenger--notifier').each(function () {
                            $(this).remove();
                        });
                    }, 500);
                }
            };
        }())
    };
    return new NotificationController();
});
