/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/core/properties', [
    'com.voiceworks/ox-messenger/util',
    'gettext!com.voiceworks/ox-messenger',
    'settings!com.voiceworks/ox-messenger'
], function (Util, gt, settings) {
    'use strict';

    var PropertiesModel = Backbone.Model.extend({

        addProperty: function (name, value) {
            var key = Util.keyify(name);
            this.set(key, value);
        },

        getProperty: function (name) {
            var key = Util.keyify(name);
            var value = this.get(key);

            if (value === undefined) {
                value = false;
            }

            return value;
        },

        deleteProperty: function (name) {
            var key = Util.keyify(name);
            this.unset(key);
        },

        updateProperty: function (oldProperty, newProperty) {
            var value = this.get(oldProperty);

            this.deleteProperty(oldProperty);
            this.addProperty(newProperty, value);
        },

        clean: function (string) {
            if (!string) {
                return string;
            }
            var cleanString = string.toLowerCase().replace(/\W/g, '_');
            return cleanString;
        },

        toArray: function () {
            var values = _.map(this.attributes, function (value, label) {
                if (value === true) { return parseInt(label, 10); }
            });

            var compactValues = _.compact(values);

            return compactValues;
        },

    });

    return PropertiesModel;
});
