/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/core/views/ext/participants-view', [
    'io.ox/participants/views',
    'com.voiceworks/ox-messenger/roster/main',
    'gettext!com.voiceworks/ox-messenger'
], function (pViews, Roster, gt) {
    'use strict';

    /**
     * Add Participant View
     */
    var ParticipantEntryViewMod = pViews.ParticipantEntryView.extend({
        // This component renders different kinds of models: OX contact models and Messenger (that's why it's messy)
        render: function() {
            var buddies         = Roster.getAllUsers(),
                keyEmail        = this.model.get(this.model.get('field')) || this.model.get('id'),
                displayEmail    = this.model.get('email2') || this.model.get('email1') || this.model.get('id'),
                user            = buddies.get(keyEmail),
                imgDiv,
                wrapperDiv,
                nameDiv,
                emailDiv;

            // If the user is not found, it "probably" means the model is the logged User
            if (!user){
                user = this.model;
            }

            imgDiv      = $('<div  class="messenger--media__img  media__img-avatar  messenger--avatar  messenger--avatar--small">').css('background-image', 'url(' + user.getImageUrl() + ')');
            wrapperDiv  = $('<div class="messenger--media__body">');
            nameDiv     = $('<div class="messenger--h5  messenger--font--bold  messenger--buddy__username  messenger--text-wrap">')
                            .text(user.getDisplayName())
                            .prepend($('<span class="messenger--presence">').addClass('is-status--' + user.get('status'))),
            emailDiv    = $('<div class="messenger--h5  messenger--buddy__status">').text(displayEmail);

            this.$el.append(
                imgDiv,
                wrapperDiv.append(nameDiv, emailDiv),
                $('<a href="#" class="remove" role="button" tabindex="1">').append(
                    $('<div class="icon">').append(
                        $('<i class="fa fa-trash-o" aria-hidden="true">'),
                        $('<span class="sr-only">').text(gt('Remove contact') + ' ' + this.model.getDisplayName())
                    )
                )
            ).attr({ 'data-cid': this.model.cid }).toggleClass('removable', this.options.closeButton);

            this.options.customize.call(this);

            return this;
        },
        setTypeStyle: function () {
            return false;
        }
    });

    var UserControllerMod = pViews.UserContainer.extend({
        renderParticipant: function (participant) {
            var self = this;
            var view = new ParticipantEntryViewMod({
                tagName: 'li',
                model: participant,
                baton: self.options.baton,
                halo: true,
                closeButton: true
            }).render().$el.addClass('col-xs-12 col-sm-6');

            setTimeout(function() {
                self.$el.find('a.remove').attr({'tabindex': 9});
            }, 0);

            // bring organizer up
            if (participant.get('id') === self.options.baton.model.get('organizerId')) {
                self.$ul.prepend(view);
            } else {
                self.$ul.append(view);
            }
        }
    });

    return {
        ParticipantEntryView: ParticipantEntryViewMod,
        UserContainer: UserControllerMod
    };
});
