/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/core/views/virtualdom', [
    'com.voiceworks/ox-messenger/core/views/disposable',
    'com.voiceworks/ox-messenger/core/vendor/virtual-dom'
], function (DisposableView, vdom) {
    'use strict';

    var diff = vdom.diff;
    var patch = vdom.patch;
    var createElement = vdom.create;
    var VNode = vdom.VNode;
    var VText = vdom.VText;

    var VirtualDomView = DisposableView.extend({
        /**
         * When set to `true`, the view element that is created by Backbone will be replaced by the element of
         * `virtualDom/createElement` function.
         */
        replaceElement: false,
        _vDomElement: null,
        _vDomTree: null,

        // Hyperscript builder
        h: vdom.h,

        initialize: function () {
            DisposableView.prototype.initialize.apply(this, arguments);
        },

        /**
         * @param {Boolean} [immediately=false]
         * @return {VirtualDomView}
         * @override
         */
        render: function (immediately) {
            this._vDomRender();

            if (!this._vDomElement) {
                this._vDomElement = createElement(this._vDomTree);

                if (this.replaceElement) {
                    this.el = this._vDomElement;
                } else {
                    this.el.appendChild(this._vDomElement);
                }
            }

            return this;
        },

        /**
         * Do a virtual dom diff and update the real DOM
         */
        _vDomRender: function () {
            var newTree = this.template(this);

            if (this._vDomTree && this._vDomElement) {
                var patches = diff(this._vDomTree, newTree);
                this._vDomElement = patch(this._vDomElement, patches);
            }

            this._vDomTree = newTree;
        },

        /**
         * Destroy the view instance.
         */
        remove: function () {
            VirtualDomView.__super__.remove.call(this);
            this._vDomElement = null;
            this._vDomTree = null;
        }
    });

    return VirtualDomView;
});
