        /**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/file/models/file-desc', [
    'io.ox/files/share/api',
    'io.ox/files/api'
], function (shareApi, fileApi) {
    'use strict';

    /**
     * File Descriptor Model
     */
    var FileDesc = Backbone.Model.extend({

        defaults: {
            created_by: -1,
            file_mimetype: '',
            file_size: -1,
            filename: '',
            folder_id: '',
            id: '',
            last_modified: -1,
            locked_until: -1,
            modified_by: -1,
            title: '',
            version: '',

            // OX Messenger attributes
            is_shared: false,
            shared_id: '',
            shared_folder_id: '',
            upload_complete: false,
            upload_cancelled: false,
            permission_denied: false,
            permissions_loaded: false,
            public_link: null
        },

        initialize: function () {
            // Deletes 'meta' as attribute and doesn't trigger any event
            this.unset('meta', {silent:true});
        },

        // Must be called before rendering message with a file descriptor
        processUserPermissions: function(user) {
            var fileId = this.get('id'),
                folderId = this.get('folder_id'),
                isShared = parseInt(this.get('created_by'), 10) !== user.get('contact')._uid;

            this.set('is_shared', isShared);

            // If it's not created by the user, it's a shared file
            if(this.get('is_shared')) {
                folderId = 10;
                fileId = fileId.replace(this.get('folder_id') + '/', folderId + '/');

                this.set('shared_id', fileId);
                this.set('shared_folder_id', folderId);
            }
        },

        // Used to render the file preview (based on permissions)
        toPreviewData: function(){
            var previewData = this.toObject(true);

            previewData.name = previewData.filename;
            previewData.filename = null; // Delete the file name (needed by OX file preview)
            previewData.dataURL = fileApi.getUrl(previewData, 'view');
            previewData.downloadURL = fileApi.getUrl(previewData, 'download');
            previewData.mimetype = previewData.file_mimetype;
            previewData.size = previewData.file_size;

            // Restore the file name
            previewData.filename = previewData.name;

            return previewData;
        },

        // Returns copy of the attributes of the fileDesc as an Object
        toObject: function(usePermissions){
            var clonedObj = this.toJSON();

            // If it's a shared file, we switch IDs
            if(usePermissions && this.get('is_shared')){
                clonedObj.id = this.get('shared_id');
                clonedObj.folder_id = this.get('shared_folder_id');
            }

            return clonedObj;
        }

    });

    return FileDesc;
});
