/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/file/views/upload-button', [
    'com.voiceworks/ox-messenger/file/main',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/file/models/file-desc',
    'gettext!com.voiceworks/ox-messenger'
], function (FileController, Aggregator, FileDesc, gt) {
    'use strict';

    /**
     * File Upload Button
     */
    var UploadButton = Backbone.View.extend({
        className: 'messenger--table__cell  messenger--table__cell--chat-input__file-upload  dropup',

        events: {
            'click .option-local':  'openLocalFilePicker',
            'click .option-drive':  'openDriveFilePicker'
        },

        render: function () {
            var circleIcon = $('<button class="dropdown-toggle messenger--btn--fa fa fa-paperclip" type="button" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false"/>'),
                optionsList = $('<ul class="dropdown-menu">'),
                options = [
                    {class: 'option-local', label: gt('Select local file')},
                    {class: 'option-drive', label: gt('Select from drive')}
                ];

            // Add options to the list
            options.forEach(function(option){
                optionsList.append(
                    $('<li><a class="'+option.class+'">'+option.label+'</a></li>')
                );
            });

            // Add sub-elements
            this.$el.empty();
            this.$el.append(circleIcon);
            this.$el.append(optionsList);

            return this;
        },

        openLocalFilePicker: function () {
            var input = $('<input type="file" name="file" capture="camera" multiple>'),
                onFileSelected;

            onFileSelected = function (event) {
                FileController.uploadFiles(
                    event.target.files,
                    function(fileDesc){
                        Aggregator.trigger('file:uploadstart', fileDesc);
                    },
                    function(fileDesc){
                        Aggregator.trigger('file:uploadend', fileDesc);
                    }
                );

                input.remove();
                event.preventDefault();
            };

            // Hide input and add file selection event
            input.hide().on('change', onFileSelected);

            // Add an invisible input element
            this.$el.after(input);

            // Simulates user's click
            input.trigger('click');
        },

        openDriveFilePicker: function () {
            require(['io.ox/files/filepicker'], function (Picker) {
                var filePicker = new Picker({
                    primaryButtonText : gt('Add'),
                    cancelButtonText  : gt('Cancel'),
                    header            : gt('Add files'),
                    multiselect       : true
                });

                // Once the file(s) are selected
                filePicker.done(function (files) {
                    files.forEach(function (fileData) {
                        // We make a copy of the file into OX Messenger Uploads
                        // to avoid permission conflicts
                        FileController
                            .copyFileToUploads(new FileDesc(fileData))
                            .done(function(fileDesc){
                                fileDesc.set('upload_complete', true);
                                Aggregator.trigger('file:picked', fileDesc);
                            });
                    });
                });
            });
        }
    });

    return UploadButton;
});
