/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/history/models/history', [
    'settings!com.voiceworks/ox-messenger'
], function (settings) {
    'use strict';

    var HistoryModel = Backbone.Model.extend({

        getOtherParty: function () {
            var caller = this.getCaller(),
                callee = this.getCallee();

            return this.isOutbound() ? callee : caller;
        },

        getCallHangupCause: function () {
            switch (this.get('hangup_cause')) {
            case 'NORMAL_CLEARING':
                return 'normal';
            case 'ORIGINATOR_CANCEL':
            case 'NO_USER_RESPONSE':
            case 'NO_ANSWER':
            case 'SUBSCRIBER_ABSENT':
            case 'USER_BUSY':
            case 'CALL_REJECTED':
                return this.isOutbound() ? 'cancelled' : 'missed';
            default:
                return 'cancelled';
            }
        },

        getStartStamp: function () {
            if (this.get('start_stamp').match(/Z$/g)) {
                return this.get('start_stamp');
            }

            // Temporary fix: if time stamp is not in UTC, add time zone (C.E.T without DST)
            return this.get('start_stamp').replace(' ', 'T') + '+01:00';
        },

        isOutbound:  function () {
            return this.get('call_direction') === 'outbound';
        },

        isPstn: function () {
            if (this.isOutbound()) {
                return this.get('effective_destination_domain').toLowerCase() === settings.get('pstnCallDomain').toLowerCase();
            } else {
                return this.get('effective_from_domain').toLowerCase() === settings.get('pstnCallDomain').toLowerCase();
            }
        },

        getCaller: function () {
            return this.isPstn() ? this.get('effective_from_user') : this.get('effective_from_user') + '@' + this.get('effective_from_domain');
        },

        getCallee: function () {
            return this.isPstn() ? this.get('effective_destination_user') : this.get('effective_destination_user') + '@' + this.get('effective_destination_domain');
        }

    });

    return HistoryModel;
});
