/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/history/models/historycollection', [
    'com.voiceworks/ox-messenger/history/models/history',
    'com.voiceworks/ox-messenger/util'
], function (HistoryModel, Util) {
    'use strict';

    var HistoryCollection = Backbone.Collection.extend({

        model: HistoryModel,

        comparator: function (cdr) {
            // Reverse sort by date
            //return new Date(cdr.get('start_stamp').replace(' ','T')).getTime() * -1;
            return Date.parse(cdr.getStartStamp()) * -1;
        },

        getPhoneHistory: function (limit) {
            var phoneHistory = this.filter(function (cdr) {
                return cdr.isPstn();
            });

            if (limit > 0) {
                var collection = new HistoryCollection(phoneHistory);
                return new HistoryCollection(collection.first(limit));
            } else {
                return new HistoryCollection(phoneHistory);
            }
        },

        getContactHistory: function (user, limit) {
            var contactHistory = this.filter(function (cdr) {
                var caller = cdr.getCaller(),
                    callee = cdr.getCallee(),
                    phoneMapping = Util.contactPhoneFieldMapping;

                // Match on-net calls
                if (caller === user.id || callee === user.id) {
                    return true;
                }

                // Match PSTN calls
                _.each(Util.contactPhoneFields, function (field) {
                    if (user.get('contact')[field] && (user.get('contact')[field] === caller || user.get('contact')[field] === callee)) {
                        return true;
                    }
                });

                return false;
            });

            var collection = new HistoryCollection(contactHistory);

            if (limit > 0) {
                return new HistoryCollection(collection.first(limit));
            } else {
                return collection;
            }
        }
    });

    return HistoryCollection;
});
