/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/history/views/small', [
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/util',
    'com.voiceworks/ox-messenger/core/session',
    'com.voiceworks/ox-messenger/core/views/disposable',
    'com.voiceworks/ox-messenger/core/contactsearch',
    'com.voiceworks/ox-messenger/roster/main',
    'gettext!com.voiceworks/ox-messenger',
], function (Aggregator, Util, Session, DisposableView, ContactSearch, Roster, gt) {
    'use strict';

    var SmallHistoryView = DisposableView.extend({

        initialize: function () {
            DisposableView.prototype.initialize.apply(this, arguments);

            this.collection = this.getHistory();
            this.listenTo(Session.user.callHistory, 'add reset', this.historyUpdated);
        },

        events: {
            'click .messenger--js-show-full-history': 'showFullHistory'
        },

        historyUpdated: function () {
            this.collection = this.getHistory();
            this.render();
        },

        render: function (options) {
            var that = this;
            this.renderOptions = options;

            this.$el.empty();
            this.$el.children().remove();
            this.$el.prevObject = null;

            this.$el.append(
                $('<div class="messenger--table__row  messenger--table__row--cdr-small  messenger--h5  messenger--font--bold">').append(
                    $('<div class="messenger--table__cell  messenger--table__cell--cdr-small  messenger--table__cell--cdr-small--icon  messenger--text-wrap">'),
                    $('<div class="messenger--table__cell  messenger--table__cell--cdr-small  messenger--table__cell--cdr-small--name  messenger--text-wrap">').text(gt('Name')),
                    $('<div class="messenger--table__cell  messenger--table__cell--cdr-small  messenger--table__cell--cdr-small--date  messenger--text-wrap">').text(gt('Date'))
                )
            );

            this.collection.each(function (cdr) {
                var type = $('<span class="messenger--table__cell  messenger--table__cell--cdr-small  messenger--table__cell--cdr-small--icon">'),
                    party = $('<span class="messenger--table__cell  messenger--table__cell--cdr-small  messenger--table__cell--cdr-small--name  messenger--text-wrap">'),
                    date = $('<span class="messenger--table__cell  messenger--table__cell--cdr-small  messenger--table__cell--cdr-small--date  messenger--text-wrap">'),
                    otherUser = cdr.getOtherParty(),
                    contacts = ContactSearch.byPhonenumber(cdr.get('effective_destination_user')),
                    link = $('<a class="messenger--table__row  messenger--table__row--cdr-small  messenger--h6  messenger--pointer" href="#">');

                // Remove link for links inside Halo
                if (that.renderOptions && that.renderOptions.halo) {
                    link = $('<div class="messenger--table__row  messenger--table__row--cdr-small  messenger--h6">');
                }

                // Type
                if (cdr.getCallHangupCause() !== 'normal') {
                    type.addClass('tmp-mes-callfailed');
                }

                if (cdr.isPstn()) {
                    type.append(
                        $('<i class="fa fa-phone">')
                        // cdr.get('call_direction') === 'outbound' ? $('<i class="fa fa-arrow-up  messenger--fa-arrow-up--outgoing">') : $('<i class="fa fa-arrow-up  messenger--fa-arrow-up--incoming">')
                    );
                } else {
                    type.append(
                        //cdr.get('type') === 'audio' ? $('<i class="fa fa-phone">') : $('<i class="fa fa-video-camera">'),
                        $('<i class="fa fa-desktop">')
                        // cdr.get('call_direction') === 'outbound' ? $('<i class="fa fa-arrow-up  messenger--fa-arrow-up--outgoing">') : $('<i class="fa fa-arrow-up  messenger--fa-arrow-up--incoming">')
                    );
                }

                // Other party
                if (cdr.isPstn()) {
                    var contacts = ContactSearch.byPhonenumber(otherUser),
                        partyName = contacts[0] ? contacts[0].name : otherUser;

                    link.on('click', function (e) {
                        e.preventDefault();
                        that.pstnAction(otherUser);
                    });

                    party.append(
                        $('<span>').text(partyName)
                    );

                } else {
                    party.append(
                        $('<span>').text(Roster.getUserById(otherUser).getDisplayName())
                    );

                    if (that.renderOptions && !that.renderOptions.halo) {
                        link.addClass('halo-link').data({ email1: otherUser });
                        // link.removeAttr('href');
                    }
                }

                // Date
                date.append(Util.formatDateTime(Date.parse(cdr.getStartStamp())));

                link.append(
                    type,
                    party,
                    date
                );

                if (cdr.getCallHangupCause() !== 'normal') {
                    link.children().addClass('messenger--table__cell--cdr-small--missed-call');
                }

                that.$el.append(link);
            });

            if (this.collection.length === 0) {
                this.$el.append(
                    $('<div class="messenger--table__cell  messenger--table__cell--cdr  messenger--table__cell--cdr--icon">').text(' '),
                    this.showEmptyMessage(),
                    $('<div class="messenger--table__cell  messenger--table__cell--cdr  messenger--table__cell--cdr--date">').text(' ')
                );
            }

            this.$el.append(
                $('<a class="btn btn-primary  messenger--btn--show-full-history  messenger--h6  messenger--js-show-full-history">').text(gt('Show full history'))
            );

            return this;
        },

        showFullHistory: function () { },

        showEmptyMessage: function () { },

        pstnAction: function () { },

        getHistory: function () { }
    });

    return SmallHistoryView;
});
