/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/phone/main', [
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/phone/views/phone',
    'gettext!com.voiceworks/ox-messenger',
    'less!com.voiceworks/ox-messenger/style'
], function (Aggregator, PhoneView, gt) {
    'use strict';

    /**
     * Main phone Application
     */
    var Phone = {

        init: function (user) {
            var that = this;
            // Remove old toggle listener
            //Aggregator.off('phone:toggle');
            Aggregator.on('phone:toggle', $.proxy(this.toggleHandler, this));
            Aggregator.on('phone:setNumber', $.proxy(this.setNumberHandler, this));
            Aggregator.on('phone:hide', $.proxy(this.hide, this));

            this.user = user;
            this.view = new PhoneView({ user: user });

            require('io.ox/core/notifications').on('show', function () {
                that.hide();
            });
        },

        attach: function () {
            var that = this;

            $('#io-ox-core').prepend(
                $('<div class="messenger--phone messenger--is-hidden" tabindex="-1">'),
                $('<div class="abs messenger--phone-overlay  messenger--is-hidden">').click(function (e) {
                    if (e.target === this) {
                        that.hide();
                    }
                })
            );
        },

        toggleHandler: function (e) {
			this.view.$el.toggleClass('messenger--is-hidden')
                .siblings('.messenger--phone-overlay').toggleClass('messenger--is-hidden');

            this.view.$el.find('.messenger--phone__component--quick__input').focus();

            this.setNumberHandler(e);

            require('io.ox/core/notifications').hide();

            // if (!this.view.$el.addClass('messenger--is-hidden')) {
            //     Aggregator.trigger('history:refresh');
            // }
        },

        setNumberHandler: function (e) {
            if (e && e.phoneNumber) {
                this.setNumber(e.phoneNumber);
                this.view.txtInputChanged();
            }
        },

        show: function () {
            this.view.$el.removeClass('messenger--is-hidden')
                .siblings('.messenger--phone-overlay').removeClass('messenger--is-hidden');
        },

        hide: function () {
            if (this.view) {
                this.view.$el.addClass('messenger--is-hidden')
                    .siblings('.messenger--phone-overlay').addClass('messenger--is-hidden');

                // Empty search field
                this.view.$el.find('.messenger--phone__component--quick__input').val('');
            }
        },

        setNumber: function (number) {
            this.view.$el.find('.messenger--phone__component--quick__input').val(number).focus();
        }
    };

    return Phone;
});
