/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/roster/models/groupcollection', [
    'com.voiceworks/ox-messenger/roster/models/group'
], function (RosterGroupModel) {
    'use strict';

    /**
     * Roster Group Collection
     */
    var RosterGroupCollection = Backbone.Collection.extend({

        model: RosterGroupModel,

        // Multiline sorting
        // http://www.benknowscode.com/2013/12/multi-column-sort-in-backbone-collections.html
        //
        sortColumn: 'index,name',
        sortDirection: 'asc',
        comparator: function (a, b) {
            if (!this.sortColumn) { return 0; }

            var cols = this.sortColumn.split(','),
                dirs = this.sortDirection.split(','),
                cmp;

            // First column that does not have equal values
            cmp = _.find(cols, function (c) {
                return a.attributes[c] != b.attributes[c];
            });

            // undefined means they're all equal, so we're done.
            if (!cmp) { return 0; }

            // Otherwise, use that column to determine the order
            // match the column sequence to the methods for ascending/descending
            // default to ascending when not defined.
            if ((dirs[_.indexOf(cols, cmp)] || 'asc').toLowerCase() == 'asc') {
                return a.attributes[cmp] > b.attributes[cmp] ? 1 : -1;
            } else {
                return a.attributes[cmp] < b.attributes[cmp] ? 1 : -1;
            }
        },
    });

    return RosterGroupCollection;
});