/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/roster/views/group', [
    'io.ox/core/extensions',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/roster/models/group',
    'com.voiceworks/ox-messenger/core/views/disposable',
    'com.voiceworks/ox-messenger/roster/views/groupheader',
    'com.voiceworks/ox-messenger/roster/views/userlist',
    'com.voiceworks/ox-messenger/util'
], function (ext, Aggregator, RosterGroupModel, DisposableView, RosterGroupHeaderView, RosterUserListView, Util) {
    'use strict';

    /**
     * Roster Group Extension points:
     */

    /**
     * Roster Group View
     *
     * this.model = RosterGroup
     * this.model.get('users') = RosterUserCollection
     */
    var RosterGroupView = DisposableView.extend({
        className: 'messenger--roster__group',

        initialize: function (options) {
            DisposableView.prototype.initialize.apply(this, arguments);

            this.user = options.user;

            this.listenTo(this.model, 'change:query', this.render);
            this.listenTo(this.model, 'change:toggled', this.toggle);

            this.listenTo(this.model.users, 'reset add remove', this.render);

            this.listenTo(Aggregator, 'roster:search', this.onSearch);
            this.listenTo(Aggregator, 'call:started', this.render);
            this.listenTo(Aggregator, 'call:ended', this.render);
        },

        onSearch: function (e) {
            var keyword = $(e.currentTarget).val(),
                trimmedKeyword = $.trim(keyword),
                escapedQuery = Util.escapeRegExp(trimmedKeyword);
            this.model.set('query', escapedQuery);
            this.user.set('rosterQuery', escapedQuery);
        },

        render: function () {
            var container = document.createDocumentFragment(),
                rosterGroupModel = this.model,
                rosterUserCollection = rosterGroupModel.users;

            // If rendered and there is still a rosterQuery in the Session User, use that
            if ((!this.model.get('query') || this.model.get('query').length === 0) && this.user.get('rosterQuery').length > 0) {
                this.model.set('query', this.user.get('rosterQuery'));
            }

            if (this.model.get('query').length > 0) {
                rosterUserCollection = this.model.search();
                rosterGroupModel = new RosterGroupModel({
                    id: this.model.id,
                    name: this.model.get('name'),
                    users: rosterUserCollection
                });
            }

            this.rosterGroupHeaderView = new RosterGroupHeaderView({
                user: this.user,
                model: rosterGroupModel,
                collection: rosterUserCollection
            });
            this.rosterUserCollectionView = new RosterUserListView({
                user: this.user,
                collection: rosterUserCollection,
                query: (this.model.get('query').length > 0) ? this.model.get('query') : null
            });

            container.appendChild(this.rosterGroupHeaderView.render().$el[0]);
            container.appendChild(this.rosterUserCollectionView.render().$el[0]);

            this.$el.children().remove();
            this.$el.prevObject = null;

            this.$el.append(container);
            container = null;

            this.toggle();

            return this;
        },

        toggle: function () {
            if (this.model.get('toggled') === true) {
                this.rosterGroupHeaderView.hide();
                this.rosterUserCollectionView.hide();
            } else {
                this.rosterGroupHeaderView.show();
                this.rosterUserCollectionView.show();
            }
        }
    });

    return RosterGroupView;
});