/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/roster/views/groupheader', [
    'io.ox/core/extensions',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/core/views/disposable',
    'com.voiceworks/ox-messenger/roster/models/group',
    'com.voiceworks/ox-messenger/roster/models/groupcollection',
    'com.voiceworks/ox-messenger/roster/views/new_group',
    'gettext!com.voiceworks/ox-messenger',
    'settings!com.voiceworks/ox-messenger',
], function (ext, Aggregator, DisposableView, RosterGroupModel, RosterGroupCollection, RosterNewGroupView, gt, settings) {
    'use strict';

    /**
     * Roster Group Header View
     *
     * this.model = RosterGroupModel
     */
    var RosterGroupHeaderView = DisposableView.extend({

        className: 'messenger--roster__group-title',

        initialize: function () {
            DisposableView.prototype.initialize.apply(this, arguments);

            this.hidden = true;

            this.model = this.model || new RosterGroupModel();
            this.collection = this.collection || new RosterGroupCollection();
            this.listenTo(this.model, 'change', this.render);
            this.listenTo(this.collection, 'reset', this.render);
            this.listenTo(this.collection, 'change:status', this.render);
        },

        events: {
            'click .messenger-js--roster__group-title__toggle': 'toggle',
            'click .messenger-js--roster__group-title__name': 'toggle',
            'click .messenger-js--roster__group-title__action-delete': 'delete',
            'click .messenger-js--roster__group-title__action-edit': 'edit'
        },

        render: function () {
            if (!this.model) {
                return;
            }

            this.$el.empty();
            this.$el.prevObject = null;

            var groupLength = settings.showOfflineContacts ? this.collection.length : this.collection.getOnlineUsers().length + '/' + this.collection.length;

            var editButton = '',
                deleteButton = '',
                toggleButton = $('<span class="messenger--roster__group-title__toggle  messenger--push-left--quarter  messenger-js--roster__group-title__toggle">');

            if (this.model.get('toggled')) {
                toggleButton.append($('<i class="fa fa-chevron-right">'));
            } else {
                toggleButton.append($('<i class="fa fa-chevron-down">'));
            }

            // Hide edit tools for Groups 1,2 & 3
            var groupId = parseInt(this.model.id, 10);

            if (groupId && [1,2,3].indexOf(groupId) === -1) {
                editButton = $('<a href="#" class="messenger--roster__group-title__action-edit  messenger-js--roster__group-title__action-edit">').append(
                    $('<i class="fa fa-pencil">')
                );
                deleteButton = $('<a href="#" class="messenger--roster__group-title__action-delete  messenger-js--roster__group-title__action-delete">').append(
                    $('<i class="fa fa-trash-o">')
                );
            }

            this.$el.append(
                $('<div class="messenger--h4  messenger--font--light">').append(
                    toggleButton,
                    $('<span class="messenger-js--roster__group-title__name">').text(this.model.get('name')),
                    $('<span class="messenger--count">').text(groupLength),
                    editButton,
                    deleteButton
                )
            );

            return this;
        },

        toggle: function (event) {
            event.preventDefault;
            event.stopPropagation();

            var object = {
                model  : this.model,
                hidden : this.hidden
            };

            Aggregator.trigger('roster:group:toggle', object);
        },

        show: function () {
            this.$el.find('.messenger-js--roster__group-title__toggle i.fa').addClass('fa-chevron-down').removeClass('fa-chevron-right');
        },

        hide: function () {
            this.$el.find('.messenger-js--roster__group-title__toggle i.fa').addClass('fa-chevron-right').removeClass('fa-chevron-down');
        },

        delete: function (event) {
            event.preventDefault();

            var object = {
                group: this.model
            };

            Aggregator.trigger('roster:group:delete', object);
        },

        edit: function (event) {
            event.preventDefault();

            this.$el.empty();

            var rosterEditGroupView = new RosterNewGroupView({
                model: this.model,
                inputValue : this.model.get('name'),
                submitType : 'update'
            });

            this.$el.append(rosterEditGroupView.render().$el);

            rosterEditGroupView.toggle();

            Aggregator.on('roster:group:edit:cancel', $.proxy(this.render, this));
        },

    });

    return RosterGroupHeaderView;
});