/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/roster/views/userlist', [
    'io.ox/core/extensions',
    'com.voiceworks/ox-messenger/roster/models/usercollection',
    'com.voiceworks/ox-messenger/core/views/disposable',
    'com.voiceworks/ox-messenger/roster/views/user',
    'settings!com.voiceworks/ox-messenger'
], function (ext, RosterUserCollection, DisposableView, RosterUserView, settings) {
    'use strict';

    /**
     * Extension points:
     */

    /**
     * View
     *
     * this.model = RosterGroup
     * this.collection = RosterUserCollection
     */
    var RosterUserListView = DisposableView.extend({

        tagName: 'ul',

        className: 'messenger--roster__list messenger--vertical-list',

        initialize: function (options) {
            DisposableView.prototype.initialize.apply(this, arguments);

            this.user = options.user;
            this.collection = this.collection || new RosterUserCollection();
            this.sortingScheduled = false;
            this.queuedSorts = 0;
            this.listenTo(this.collection, 'reset', this.render);
            this.listenTo(this.collection, 'sort', this.sortingScheduler);
            this.contactsListener = settings.on('change:showOfflineContacts', $.proxy(this.render, this));
            this.query = options.query;
        },

        sortingScheduler: function () {
            var that = this;

            if (that.sortingScheduled === false) {
                that.sortingScheduled = true;
                that.render();

                setTimeout(function () {
                    if (that.queuedSorts > 0) {
                        that.render();
                    }
                    that.sortingScheduled = false;
                    that.queuedSorts = 0;
                }, 1000);
            } else {
                that.queuedSorts++;
            }
        },

        render: function () {
            var that = this,
                container = document.createDocumentFragment();

            if (!this.collection) { return this; }

            this.collection.each(function (user) {
                // Whether or not to show offline buddies is determined in the Messenger settings, always show all contacts that match the search query
                if (!that.query && !settings.get('showOfflineContacts') && user.get('status') === 'offline') {
                    return;
                }

                var rosterUserView = new RosterUserView({ user: that.user, model: user });
                container.appendChild(rosterUserView.render().$el[0]);
            });

            this.$el.children().remove();
            this.$el.prevObject = null;

            this.$el.append(container);
            container = null;

            return this;
        },

        show: function () {
            this.$el.removeClass('messenger--is-hidden');
        },

        hide: function () {
            this.$el.addClass('messenger--is-hidden');
        },

    });

    return RosterUserListView;
});