define(['io.ox/core/extensions',
        'com.voiceworks/ox-messenger/roster/views/roster',
        'com.voiceworks/ox-messenger/roster/models/group',
        'com.voiceworks/ox-messenger/core/sessionuser',
        'com.voiceworks/ox-messenger/roster/models/groupcollection'],
    function (ext, RosterView, RosterGroupModel, SessionUser, RosterGroupCollection) {

    describe('RosterView ', function () {
        var roster;

        beforeEach(function () {
            var timestamp = (new Date().getTime()),
                fakemodel = new RosterGroupCollection(),
                user = new SessionUser();
            roster = new RosterView({collection: fakemodel, timestamp: timestamp, user: user});
        });

        it('should contain RosterMyStatusView', function () {
            var html = roster.render(),
                hasRightClass = html.$('.messenger--roster__mystatus');
            expect(hasRightClass).to.be.ok;
        });

        it('should contain SearchView', function () {
            var html = roster.render(),
                hasRightClass = html.$('.messenger--roster__search');
            expect(hasRightClass).to.be.ok;
        });


        it('should contain GroupCollectionView', function () {
            var html = roster.render(),
                hasRightClass = html.$('.messenger--roster__group');
            expect(hasRightClass).to.be.ok;
        });


        it('should invoke an extension point for the status', function (done) {
            ext.point('io.ox/messenger/roster/mystatus').extend({
                id: 'roster-status-test',
                draw: function () {
                    done();
                }
            });
            roster.render();
        });


        it('should invoke an extension point for the search', function (done) {
            ext.point('io.ox/messenger/roster/search').extend({
                id: 'roster-search-test',
                draw: function () {
                    done();
                }
            });
            roster.render();
        });


        it('should invoke an extension point for the user list', function (done) {
            ext.point('io.ox/messenger/roster/user').extend({
                id: 'roster-userlist-test',
                draw: function () {
                    done();
                }
            });
            roster.render();
        });


        it('should invoke an extension point for the header', function (done) {
            ext.point('io.ox/messenger/roster/group/header').extend({
                id: 'roster-header-test',
                draw: function () {
                    done();
                }
            });
            roster.render();
        });
    });

});
